@@ -24,13 +24,14 @@
 */
 
 #include "akregatorconfig.h"
+#include "feed.h"
 #include "article.h"
 #include "articlejobs.h"
-#include "feed.h"
 #include "feediconmanager.h"
 #include "feedstorage.h"
 #include "fetchqueue.h"
 #include "folder.h"
+#include "notificationmanager.h"
 #include "storage.h"
 #include "treenodevisitor.h"
 #include "types.h"
@@ -57,6 +58,8 @@
 
 #include <boost/bind.hpp>
 
+#include <memory>
+
 using Syndication::ItemPtr;
 using namespace Akregator;
 using namespace boost;
@@ -231,7 +234,7 @@ void Feed::loadArticles()
         d->archive = d->storage->archiveFor(xmlUrl());
 
     QStringList list = d->archive->articles();
-    for ( QStringList::ConstIterator it = list.begin(); it != list.end(); ++it)
+    for ( QStringList::ConstIterator it = list.constBegin(); it != list.constEnd(); ++it)
     {
         Article mya(*it, this);
         d->articles[mya.guid()] = mya;
@@ -247,14 +250,14 @@ void Feed::loadArticles()
 void Feed::recalcUnreadCount()
 {
     QList<Article> tarticles = articles();
-    QList<Article>::Iterator it;
-    QList<Article>::Iterator en = tarticles.end();
+    QList<Article>::ConstIterator it;
+    QList<Article>::ConstIterator en = tarticles.constEnd();
 
     int oldUnread = d->archive->unread();
 
     int unread = 0;
 
-    for (it = tarticles.begin(); it != en; ++it)
+    for (it = tarticles.constBegin(); it != en; ++it)
         if (!(*it).isDeleted() && (*it).status() != Read)
             ++unread;
 
@@ -342,7 +345,7 @@ void Feed::setMarkImmediatelyAsRead(bool enabled)
 {
     d->markImmediatelyAsRead = enabled;
     if (enabled)
-        slotMarkAllArticlesAsRead();
+        createMarkAsReadJob()->start();
 }
 
 void Feed::setUseNotification(bool enabled)
@@ -372,6 +375,8 @@ QString Feed::xmlUrl() const { return d->xmlUrl; }
 void Feed::setXmlUrl(const QString& s)
 {
     d->xmlUrl = s;
+    if( ! Settings::fetchOnStartup() )
+        QTimer::singleShot(KRandom::random() % 4000, this, SLOT(slotAddFeedIconListener())); // TODO: let's give a gui some time to show up before starting the fetch when no fetch on startup is used. replace this with something proper later...
 }
 
 QString Feed::htmlUrl() const { return d->htmlUrl; }
@@ -409,25 +414,23 @@ QDomElement Feed::toOPML( QDomElement parent, QDomDocument document ) const
     if (d->loadLinkedWebsite)
         el.setAttribute( "loadLinkedWebsite", "true" );
     el.setAttribute( "maxArticleNumber", d->maxArticleNumber );
-    el.setAttribute( "type", "rss" ); // despite some additional fields, its still "rss" OPML
+    el.setAttribute( "type", "rss" ); // despite some additional fields, it is still "rss" OPML
     el.setAttribute( "version", "RSS" );
     parent.appendChild( el );
     return el;
 }
 
-void Feed::slotMarkAllArticlesAsRead()
+KJob* Feed::createMarkAsReadJob()
 {
-    if (unread() > 0)
+    std::auto_ptr<ArticleModifyJob> job( new ArticleModifyJob );
+    Q_FOREACH ( const Article& i, articles() )
     {
-        ArticleModifyJob* job = new ArticleModifyJob;
-        Q_FOREACH ( const Article& i, articles() )
-        {
-            const ArticleId aid = { xmlUrl(), i.guid() };
-            job->setStatus( aid, Read );
-        }
-        job->start();
+        const ArticleId aid = { xmlUrl(), i.guid() };
+        job->setStatus( aid, Read );
     }
+    return job.release();
 }
+
 void Feed::slotAddToFetchQueue(FetchQueue* queue, bool intervalFetchOnly)
 {
     if (!intervalFetchOnly)
@@ -460,10 +463,11 @@ void Feed::appendArticles(const Syndication::FeedPtr feed)
 {
     d->setTotalCountDirty();
     bool changed = false;
+    const bool notify = useNotification() || Settings::useNotifications();
 
     QList<ItemPtr> items = feed->items();
-    QList<ItemPtr>::ConstIterator it = items.begin();
-    QList<ItemPtr>::ConstIterator en = items.end();
+    QList<ItemPtr>::ConstIterator it = items.constBegin();
+    QList<ItemPtr>::ConstIterator en = items.constEnd();
 
 
     int nudge=0;
@@ -484,7 +488,8 @@ void Feed::appendArticles(const Syndication::FeedPtr feed)
                 mya.setStatus(New);
             else
                 mya.setStatus(Read);
-
+            if ( notify )
+                NotificationManager::self()->slotNotifyArticle( mya );
             changed = true;
         }
         else // article is in list
@@ -511,9 +516,10 @@ void Feed::appendArticles(const Syndication::FeedPtr feed)
         }
     }
 
-    QList<Article>::ConstIterator dit = deletedArticles.begin();
+
+    QList<Article>::ConstIterator dit = deletedArticles.constBegin();
     QList<Article>::ConstIterator dtmp;
-    QList<Article>::ConstIterator den = deletedArticles.end();
+    QList<Article>::ConstIterator den = deletedArticles.constEnd();
 
     // delete articles with delete flag set completely from archive, which aren't in the current feed source anymore
     while (dit != den)
@@ -635,13 +641,14 @@ void Feed::fetchCompleted(Syndication::Loader *l, Syndication::FeedPtr doc, Synd
             d->fetchErrorCode = status;
             emit fetchError(this);
         }
+        markAsFetchedNow();
         return;
     }
 
     loadArticles(); // TODO: make me fly: make this delayed
 
     FeedIconManager::self()->addListener( KUrl( xmlUrl() ), this );
-    
+
     d->fetchErrorCode = Syndication::Success;
 
     if (d->imagePixmap.isNull())
@@ -651,7 +658,7 @@ void Feed::fetchCompleted(Syndication::Loader *l, Syndication::FeedPtr doc, Synd
                                 + Utils::fileNameForUrl(d->xmlUrl) + ".png";
         d->imagePixmap=QPixmap(imageFileName, "PNG");
 
-        // if we aint got teh image and the feed provides one, get it....
+        // if we ain't got the image and the feed provides one, get it....
         // TODO: reenable image fetching!
         if (false) // d->imagePixmap.isNull() && doc.image())
         {
@@ -662,17 +669,22 @@ void Feed::fetchCompleted(Syndication::Loader *l, Syndication::FeedPtr doc, Synd
     }
 
     if (title().isEmpty())
-        setTitle( doc->title() );
+        setTitle( Syndication::htmlToPlainText( doc->title() ) );
 
     d->description = doc->description();
     d->htmlUrl = doc->link();
 
     appendArticles(doc);
 
-    d->archive->setLastFetch( QDateTime::currentDateTime().toTime_t());
+    markAsFetchedNow();
     emit fetched(this);
 }
 
+void Feed::markAsFetchedNow()
+{
+    d->archive->setLastFetch( QDateTime::currentDateTime().toTime_t());
+}
+
 QIcon Feed::icon() const
 {
     if ( fetchErrorOccurred() )
@@ -773,7 +785,7 @@ void Feed::setArticleChanged(Article& a, int oldStatus)
 int Feed::totalCount() const
 {
     if ( d->totalCount == -1 )
-        d->totalCount = std::count_if( d->articles.begin(), d->articles.end(), !bind( &Article::isDeleted, _1 ) );
+        d->totalCount = std::count_if( d->articles.constBegin(), d->articles.constEnd(), !bind( &Article::isDeleted, _1 ) );
     return d->totalCount;
 }
 
@@ -856,8 +868,7 @@ void Feed::enforceLimitArticleNumber()
     int c = 0;
     const bool useKeep = Settings::doNotExpireImportantArticles();
 
-    ArticleDeleteJob* job = new ArticleDeleteJob;
-    Q_FOREACH ( const Article& i, articles )
+    Q_FOREACH ( Article i, articles )
     {
         if (c < limit)
         {
@@ -866,11 +877,9 @@ void Feed::enforceLimitArticleNumber()
         }
         else if ( !useKeep || !i.keep() )
         {
-            const ArticleId aid = { i.feed()->xmlUrl(), i.guid() };
-            job->appendArticleId( aid );
+            i.setDeleted();
         }
     }
-    job->start();
 }
 
 #include "feed.moc"
