@@ -1,7 +1,7 @@
 /*
  *  editdlgtypes.cpp  -  dialogs to create or edit alarm or alarm template types
  *  Program:  kalarm
- *  Copyright © 2001-2009 by David Jarvie <djarvie@kde.org>
+ *  Copyright © 2001-2010 by David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -40,6 +40,7 @@
 #include "reminder.h"
 #include "shellprocess.h"
 #include "soundpicker.h"
+#include "sounddlg.h"
 #include "specialactions.h"
 #include "templatepickdlg.h"
 #include "timespinbox.h"
@@ -103,7 +104,7 @@ QString EditDisplayAlarmDlg::i18n_chk_ConfirmAck()    { return i18nc("@option:ch
 *   event   != to initialise the dialog to show the specified event's data.
 */
 EditDisplayAlarmDlg::EditDisplayAlarmDlg(bool Template, bool newAlarm, QWidget* parent, GetResourceType getResource)
-	: EditAlarmDlg(Template, KAEvent::MESSAGE, parent, getResource),
+	: EditAlarmDlg(Template, KAEventData::MESSAGE, parent, getResource),
 	  mSpecialActionsButton(0),
 	  mReminderDeferral(false),
 	  mReminderArchived(false)
@@ -258,11 +259,11 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
 	lateCancel()->showAutoClose(true);
 	if (event)
 	{
-		if (mAlarmType == KAEvent::MESSAGE  &&  event->kmailSerialNumber()
+		if (mAlarmType == KAEventData::MESSAGE  &&  event->kmailSerialNumber()
 		&&  AlarmText::checkIfEmail(event->cleanText()))
 			mKMailSerialNumber = event->kmailSerialNumber();
 		lateCancel()->setAutoClose(event->autoClose());
-		if (event->defaultFont())
+		if (event->useDefaultFont())
 			mFontColourButton->setDefaultFont();
 		else
 			mFontColourButton->setFont(event->font());
@@ -272,15 +273,18 @@ void EditDisplayAlarmDlg::type_initValues(const KAEvent* event)
 		mConfirmAck->setChecked(event->confirmAck());
 		bool recurs = event->recurs();
 		int reminderMins = event->reminder();
-		if (!reminderMins  &&  event->reminderDeferral()  &&  !recurs)
+		if (!reminderMins)
 		{
-			reminderMins = event->reminderDeferral();
-			mReminderDeferral = true;
-		}
-		if (!reminderMins  &&  event->reminderArchived()  &&  recurs)
-		{
-			reminderMins = event->reminderArchived();
-			mReminderArchived = true;
+			if (event->reminderDeferral()  &&  !recurs)
+			{
+				reminderMins = event->deferDateTime().minsTo(event->mainDateTime());
+				mReminderDeferral = true;
+			}
+			else if (event->reminderArchived()  &&  recurs)
+			{
+				reminderMins = event->reminderArchived();
+				mReminderArchived = true;
+			}
 		}
 		reminder()->setMinutes(reminderMins, dateOnly());
 		reminder()->setOnceOnly(event->reminderOnceOnly());
@@ -328,17 +332,7 @@ void EditDisplayAlarmDlg::type_showOptions(bool more)
 	if (mSpecialActionsButton)
 	{
 		if (more)
-		{
-			switch (mTypeCombo->currentIndex())
-			{
-				case tFILE:
-				case tTEXT:
-					mSpecialActionsButton->show();
-					break;
-				default:
-					break;
-			}
-		}
+			mSpecialActionsButton->show();
 		else
 			mSpecialActionsButton->hide();
 	}
@@ -368,21 +362,21 @@ void EditDisplayAlarmDlg::setColours(const QColor& fgColour, const QColor& bgCol
 /******************************************************************************
 * Set the dialog's action and the action's text.
 */
-void EditDisplayAlarmDlg::setAction(KAEvent::Action action, const AlarmText& alarmText)
+void EditDisplayAlarmDlg::setAction(KAEventData::Action action, const AlarmText& alarmText)
 {
 	QString text = alarmText.displayText();
 	switch (action)
 	{
-		case KAEvent::MESSAGE:
+		case KAEventData::MESSAGE:
 			mTypeCombo->setCurrentIndex(tTEXT);
 			mTextMessageEdit->setPlainText(text);
 			mKMailSerialNumber = alarmText.isEmail() ? alarmText.kmailSerialNumber() : 0;
 			break;
-		case KAEvent::FILE:
+		case KAEventData::FILE:
 			mTypeCombo->setCurrentIndex(tFILE);
 			mFileMessageEdit->setText(text);
 			break;
-		case KAEvent::COMMAND:
+		case KAEventData::COMMAND:
 			mTypeCombo->setCurrentIndex(tCOMMAND);
 			mCmdEdit->setText(alarmText);
 			break;
@@ -526,17 +520,17 @@ bool EditDisplayAlarmDlg::type_stateChanged() const
 */
 void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, const QString& text, int lateCancel, bool trial)
 {
-	KAEvent::Action type;
+	KAEventData::Action type;
 	switch (mTypeCombo->currentIndex())
 	{
-		case tFILE:     type = KAEvent::FILE; break;
-		case tCOMMAND:  type = KAEvent::COMMAND; break;
+		case tFILE:     type = KAEventData::FILE; break;
+		case tCOMMAND:  type = KAEventData::COMMAND; break;
 		default:
-		case tTEXT:     type = KAEvent::MESSAGE; break;
+		case tTEXT:     type = KAEventData::MESSAGE; break;
 	}
 	event.set(dt, text, mFontColourButton->bgColour(), mFontColourButton->fgColour(), mFontColourButton->font(),
 	          type, lateCancel, getAlarmFlags());
-	if (type == KAEvent::MESSAGE)
+	if (type == KAEventData::MESSAGE)
 	{
 		if (AlarmText::checkIfEmail(text))
 			event.setKMailSerialNumber(mKMailSerialNumber);
@@ -545,7 +539,7 @@ void EditDisplayAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, con
 	int   fadeSecs;
 	float volume = mSoundPicker->volume(fadeVolume, fadeSecs);
 	event.setAudioFile(mSoundPicker->file().prettyUrl(), volume, fadeVolume, fadeSecs);
-	if (!trial)
+	if (!trial  &&  reminder()->isEnabled())
 		event.setReminder(reminder()->minutes(), reminder()->isOnceOnly());
 	if (mSpecialActionsButton  &&  mSpecialActionsButton->isEnabled())
 		event.setActions(mSpecialActionsButton->preAction(), mSpecialActionsButton->postAction(), mSpecialActionsButton->cancelOnError());
@@ -583,8 +577,6 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 			mCmdEdit->hide();
 			mTextMessageEdit->show();
 			mSoundPicker->showSpeak(true);
-			if (mSpecialActionsButton  &&  showingMore())
-				mSpecialActionsButton->show();
 			setButtonWhatsThis(Try, i18nc("@info:whatsthis", "Display the alarm message now"));
 			focus = mTextMessageEdit;
 			break;
@@ -594,8 +586,6 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 			mFilePadding->show();
 			mCmdEdit->hide();
 			mSoundPicker->showSpeak(false);
-			if (mSpecialActionsButton  &&  showingMore())
-				mSpecialActionsButton->show();
 			setButtonWhatsThis(Try, i18nc("@info:whatsthis", "Display the file now"));
 			mFileMessageEdit->setNoSelect();
 			focus = mFileMessageEdit;
@@ -606,8 +596,6 @@ void EditDisplayAlarmDlg::slotAlarmTypeChanged(int index)
 			slotCmdScriptToggled(mCmdEdit->isScript());  // show/hide mFilePadding
 			mCmdEdit->show();
 			mSoundPicker->showSpeak(true);
-			if (mSpecialActionsButton)
-				mSpecialActionsButton->hide();
 			setButtonWhatsThis(Try, i18nc("@info:whatsthis", "Display the command output now"));
 			focus = mCmdEdit;
 			break;
@@ -626,7 +614,7 @@ void EditDisplayAlarmDlg::slotPickFile()
 	                                  defaultDir, mFileMessageEdit->text(), QString(), KFile::ExistingOnly, this);
 	if (!file.isEmpty())
 	{
-		mFileMessageEdit->setText(file);
+		mFileMessageEdit->setText(KAlarm::pathOrUrl(file));
 		contentsChanged();
 	}
 }
@@ -657,45 +645,9 @@ bool EditDisplayAlarmDlg::checkText(QString& result, bool showErrorMessage) cons
 		case tFILE:
 		{
 			QString alarmtext = mFileMessageEdit->text().trimmed();
-			// Convert any relative file path to absolute
-			// (using home directory as the default)
-			enum Err { NONE = 0, BLANK, NONEXISTENT, DIRECTORY, UNREADABLE, NOT_TEXT_IMAGE };
-			Err err = NONE;
 			KUrl url;
-			int i = alarmtext.indexOf(QLatin1Char('/'));
-			if (i > 0  &&  alarmtext[i - 1] == QLatin1Char(':'))
-			{
-				url = alarmtext;
-				url.cleanPath();
-				alarmtext = url.prettyUrl();
-				KIO::UDSEntry uds;
-				if (!KIO::NetAccess::stat(url, uds, MainWindow::mainMainWindow()))
-					err = NONEXISTENT;
-				else
-				{
-					KFileItem fi(uds, url);
-					if (fi.isDir())             err = DIRECTORY;
-					else if (!fi.isReadable())  err = UNREADABLE;
-				}
-			}
-			else if (alarmtext.isEmpty())
-				err = BLANK;    // blank file name
-			else
-			{
-				// It's a local file - convert to absolute path & check validity
-				QFileInfo info(alarmtext);
-				QDir::setCurrent(QDir::homePath());
-				alarmtext = info.absoluteFilePath();
-				url.setPath(alarmtext);
-				alarmtext = QLatin1String("file:") + alarmtext;
-				if (!err)
-				{
-					if      (info.isDir())        err = DIRECTORY;
-					else if (!info.exists())      err = NONEXISTENT;
-					else if (!info.isReadable())  err = UNREADABLE;
-				}
-			}
-			if (!err)
+			KAlarm::FileErr err = KAlarm::checkFileExists(alarmtext, url);
+			if (err == KAlarm::FileErr_None)
 			{
 				switch (KAlarm::fileType(KFileItem(KFileItem::Unknown, KFileItem::Unknown, url).mimeTypePtr()))
 				{
@@ -705,36 +657,23 @@ bool EditDisplayAlarmDlg::checkText(QString& result, bool showErrorMessage) cons
 					case KAlarm::Image:
 						break;
 					default:
-						err = NOT_TEXT_IMAGE;
+						err = KAlarm::FileErr_NotTextImage;
 						break;
 				}
 			}
-			if (err  &&  showErrorMessage)
+			if (err != KAlarm::FileErr_None  &&  showErrorMessage)
 			{
 				mFileMessageEdit->setFocus();
-				QString errmsg;
-				switch (err)
-				{
-					case BLANK:
-						KMessageBox::sorry(const_cast<EditDisplayAlarmDlg*>(this), i18nc("@info", "Please select a file to display"));
-						return false;
-					case NONEXISTENT:     errmsg = i18nc("@info", "<filename>%1</filename> not found", alarmtext);  break;
-					case DIRECTORY:       errmsg = i18nc("@info", "<filename>%1</filename> is a folder", alarmtext);  break;
-					case UNREADABLE:      errmsg = i18nc("@info", "<filename>%1</filename> is not readable", alarmtext);  break;
-					case NOT_TEXT_IMAGE:  errmsg = i18nc("@info", "<filename>%1</filename> appears not to be a text or image file", alarmtext);  break;
-					case NONE:
-					default:
-						break;
-				}
-				if (KMessageBox::warningContinueCancel(const_cast<EditDisplayAlarmDlg*>(this), errmsg)
-				    == KMessageBox::Cancel)
+				if (!KAlarm::showFileErrMessage(alarmtext, err, KAlarm::FileErr_BlankDisplay, const_cast<EditDisplayAlarmDlg*>(this)))
 					return false;
 			}
 			result = alarmtext;
 			break;
 		}
 		case tCOMMAND:
-			result = mCmdEdit->text();
+			result = mCmdEdit->text(const_cast<EditDisplayAlarmDlg*>(this), showErrorMessage);
+			if (result.isEmpty())
+				return false;
 			break;
 	}
 	return true;
@@ -759,7 +698,7 @@ QString EditCommandAlarmDlg::i18n_chk_ExecInTermWindow()   { return i18nc("@opti
 *   event   != to initialise the dialog to show the specified event's data.
 */
 EditCommandAlarmDlg::EditCommandAlarmDlg(bool Template, bool newAlarm, QWidget* parent, GetResourceType getResource)
-	: EditAlarmDlg(Template, KAEvent::COMMAND, parent, getResource)
+	: EditAlarmDlg(Template, KAEventData::COMMAND, parent, getResource)
 {
 	kDebug() << "New";
 	init(0, newAlarm);
@@ -892,9 +831,10 @@ void EditCommandAlarmDlg::type_showOptions(bool more)
 /******************************************************************************
 * Set the dialog's action and the action's text.
 */
-void EditCommandAlarmDlg::setAction(KAEvent::Action action, const AlarmText& alarmText)
+void EditCommandAlarmDlg::setAction(KAEventData::Action action, const AlarmText& alarmText)
 {
-	Q_ASSERT(action == KAEvent::COMMAND);
+	Q_UNUSED(action);
+	Q_ASSERT(action == KAEventData::COMMAND);
 	mCmdEdit->setText(alarmText);
 }
 
@@ -949,7 +889,7 @@ bool EditCommandAlarmDlg::type_stateChanged() const
 void EditCommandAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, const QString& text, int lateCancel, bool trial)
 {
 	Q_UNUSED(trial);
-	event.set(dt, text, QColor(), QColor(), QFont(), KAEvent::COMMAND, lateCancel, getAlarmFlags());
+	event.set(dt, text, QColor(), QColor(), QFont(), KAEventData::COMMAND, lateCancel, getAlarmFlags());
 	if (mCmdOutputGroup->checkedButton() == mCmdLogToFile)
 		event.setLogFile(mCmdLogFileEdit->text());
 }
@@ -1032,8 +972,9 @@ void EditCommandAlarmDlg::slotCmdScriptToggled(bool on)
 */
 bool EditCommandAlarmDlg::checkText(QString& result, bool showErrorMessage) const
 {
-	Q_UNUSED(showErrorMessage);
-	result = mCmdEdit->text();
+	result = mCmdEdit->text(const_cast<EditCommandAlarmDlg*>(this), showErrorMessage);
+	if (result.isEmpty())
+		return false;
 	return true;
 }
 
@@ -1054,7 +995,7 @@ QString EditEmailAlarmDlg::i18n_chk_CopyEmailToSelf()    { return i18nc("@option
 *   event   != to initialise the dialog to show the specified event's data.
 */
 EditEmailAlarmDlg::EditEmailAlarmDlg(bool Template, bool newAlarm, QWidget* parent, GetResourceType getResource)
-	: EditAlarmDlg(Template, KAEvent::EMAIL, parent, getResource),
+	: EditAlarmDlg(Template, KAEventData::EMAIL, parent, getResource),
 	  mEmailRemoveButton(0)
 {
 	kDebug() << "New";
@@ -1222,9 +1163,10 @@ void EditEmailAlarmDlg::attachmentEnable()
 /******************************************************************************
 * Set the dialog's action and the action's text.
 */
-void EditEmailAlarmDlg::setAction(KAEvent::Action action, const AlarmText& alarmText)
+void EditEmailAlarmDlg::setAction(KAEventData::Action action, const AlarmText& alarmText)
 {
-	Q_ASSERT(action == KAEvent::EMAIL);
+	Q_UNUSED(action);
+	Q_ASSERT(action == KAEventData::EMAIL);
 	if (alarmText.isEmail())
 	{
 		mEmailToEdit->setText(alarmText.to());
@@ -1327,7 +1269,7 @@ bool EditEmailAlarmDlg::type_stateChanged() const
 void EditEmailAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, const QString& text, int lateCancel, bool trial)
 {
 	Q_UNUSED(trial);
-	event.set(dt, text, QColor(), QColor(), QFont(), KAEvent::EMAIL, lateCancel, getAlarmFlags());
+	event.set(dt, text, QColor(), QColor(), QFont(), KAEventData::EMAIL, lateCancel, getAlarmFlags());
 	uint from = mEmailFromList ? mEmailFromList->currentIdentity() : 0;
 	event.setEmail(from, mEmailAddresses, mEmailSubjectEdit->text(), mEmailAttachments);
 }
@@ -1338,7 +1280,7 @@ void EditEmailAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, const
 int EditEmailAlarmDlg::getAlarmFlags() const
 {
 	return EditAlarmDlg::getAlarmFlags()
-	     | (mEmailBcc->isChecked() ? KAEvent::EMAIL_BCC : 0);
+	     | (mEmailBcc->isChecked() ? KAEventData::EMAIL_BCC : 0);
 }
 
 /******************************************************************************
@@ -1469,6 +1411,180 @@ bool EditEmailAlarmDlg::checkText(QString& result, bool showErrorMessage) const
 }
 
 
+/*=============================================================================
+= Class EditAudioAlarmDlg
+= Dialog to edit audio alarms with no display window.
+=============================================================================*/
+
+/******************************************************************************
+* Constructor.
+* Parameters:
+*   Template = true to edit/create an alarm template
+*            = false to edit/create an alarm.
+*   event   != to initialise the dialog to show the specified event's data.
+*/
+EditAudioAlarmDlg::EditAudioAlarmDlg(bool Template, bool newAlarm, QWidget* parent, GetResourceType getResource)
+	: EditAlarmDlg(Template, KAEventData::AUDIO, parent, getResource)
+{
+	kDebug() << "New";
+	init(0, newAlarm);
+}
+
+EditAudioAlarmDlg::EditAudioAlarmDlg(bool Template, const KAEvent* event, bool newAlarm, QWidget* parent,
+                                     GetResourceType getResource, bool readOnly)
+	: EditAlarmDlg(Template, event, parent, getResource, readOnly)
+{
+	kDebug() << "Event.id()";
+	init(event, newAlarm);
+}
+
+/******************************************************************************
+* Return the window caption.
+*/
+QString EditAudioAlarmDlg::type_caption(bool newAlarm) const
+{
+	return isTemplate() ? (newAlarm ? i18nc("@title:window", "New Audio Alarm Template") : i18nc("@title:window", "Edit Audio Alarm Template"))
+	                    : (newAlarm ? i18nc("@title:window", "New Audio Alarm") : i18nc("@title:window", "Edit Audio Alarm"));
+}
+
+/******************************************************************************
+* Set up the dialog controls common to display alarms.
+*/
+void EditAudioAlarmDlg::type_init(QWidget* parent, QVBoxLayout* frameLayout)
+{
+	// File name edit box
+	mSoundConfig = new SoundWidget(false, true, parent);
+	if (isTemplate())
+		mSoundConfig->setAllowEmptyFile();
+	connect(mSoundConfig, SIGNAL(changed()), SLOT(contentsChanged()));
+	frameLayout->addWidget(mSoundConfig);
+
+	// Top-adjust the controls
+	mPadding = new KHBox(parent);
+	mPadding->setMargin(0);
+	frameLayout->addWidget(mPadding);
+	frameLayout->setStretchFactor(mPadding, 1);
+}
+
+/******************************************************************************
+* Initialise the dialog controls from the specified event.
+*/
+void EditAudioAlarmDlg::type_initValues(const KAEvent* event)
+{
+	if (event)
+	{
+		mSoundConfig->set(event->audioFile(), event->soundVolume(), event->fadeVolume(), event->fadeSeconds(),
+		                  (event->flags() & KAEvent::REPEAT_SOUND));
+	}
+	else
+	{
+		// Set the values to their defaults
+		mSoundConfig->set(Preferences::defaultSoundFile(), Preferences::defaultSoundVolume());
+	}
+}
+
+/******************************************************************************
+* Initialise various values in the New Alarm dialogue.
+*/
+void EditAudioAlarmDlg::setAudio(const QString& file, float volume)
+{
+	mSoundConfig->set(file, volume);
+}
+
+/******************************************************************************
+* Set the dialog's action and the action's text.
+*/
+void EditAudioAlarmDlg::setAction(KAEventData::Action action, const AlarmText& alarmText)
+{
+	Q_UNUSED(action);
+	Q_ASSERT(action == KAEventData::AUDIO);
+	mSoundConfig->set(alarmText.displayText(), Preferences::defaultSoundVolume());
+}
+
+/******************************************************************************
+* Set the read-only status of all non-template controls.
+*/
+void EditAudioAlarmDlg::setReadOnly(bool readOnly)
+{
+	mSoundConfig->setReadOnly(readOnly);
+	EditAlarmDlg::setReadOnly(readOnly);
+}
+
+/******************************************************************************
+* Save the state of all controls.
+*/
+void EditAudioAlarmDlg::saveState(const KAEvent* event)
+{
+	EditAlarmDlg::saveState(event);
+	mSavedFile   = mSoundConfig->fileName();
+	mSavedRepeat = mSoundConfig->getVolume(mSavedVolume, mSavedFadeVolume, mSavedFadeSeconds);
+}
+
+/******************************************************************************
+* Check whether any of the controls has changed state since the dialog was
+* first displayed.
+* Reply = true if any controls have changed, or if it's a new event.
+*       = false if no controls have changed.
+*/
+bool EditAudioAlarmDlg::type_stateChanged() const
+{
+        if (mSavedFile != mSoundConfig->fileName())
+                return true;
+        if (!mSavedFile.isEmpty()  ||  isTemplate())
+        {
+                float volume, fadeVolume;
+                int   fadeSecs;
+		if (mSavedRepeat      != mSoundConfig->getVolume(volume, fadeVolume, fadeSecs)
+                ||  mSavedVolume      != volume
+                ||  mSavedFadeVolume  != fadeVolume
+                ||  mSavedFadeSeconds != fadeSecs)
+                        return true;
+	}
+	return false;
+}
+
+/******************************************************************************
+* Extract the data in the dialog specific to the alarm type and set up a
+* KAEvent from it.
+*/
+void EditAudioAlarmDlg::type_setEvent(KAEvent& event, const KDateTime& dt, const QString& text, int lateCancel, bool trial)
+{
+	Q_UNUSED(text);
+	Q_UNUSED(trial);
+	event.set(dt, QString(), QColor(), QColor(), QFont(), KAEventData::AUDIO, lateCancel, getAlarmFlags());
+	float volume, fadeVolume;
+	int   fadeSecs;
+	mSoundConfig->getVolume(volume, fadeVolume, fadeSecs);
+	KUrl url;
+	mSoundConfig->file(url, false);
+	event.setAudioFile(url.prettyUrl(), volume, fadeVolume, fadeSecs, isTemplate());
+}
+
+/******************************************************************************
+* Get the currently specified alarm flag bits.
+*/
+int EditAudioAlarmDlg::getAlarmFlags() const
+{
+	return EditAlarmDlg::getAlarmFlags()
+	     | (mSoundConfig->getRepeat() ? KAEvent::REPEAT_SOUND : 0);
+}
+
+/******************************************************************************
+* Check whether the file name is valid.
+*/
+bool EditAudioAlarmDlg::checkText(QString& result, bool showErrorMessage) const
+{
+	KUrl url;
+	if (!mSoundConfig->file(url, showErrorMessage))
+	{
+		result.clear();
+		return false;
+	}
+	result = url.pathOrUrl();
+	return true;
+}
+
+
 /*=============================================================================
 = Class CommandEdit
 = A widget to allow entry of a command or a command script.
@@ -1523,7 +1639,7 @@ void CommandEdit::setText(const AlarmText& alarmText)
 	if (script)
 		mScriptEdit->setPlainText(text);
 	else
-		mCommandEdit->setText(text);
+		mCommandEdit->setText(KAlarm::pathOrUrl(text));
 }
 
 /******************************************************************************
@@ -1539,6 +1655,19 @@ QString CommandEdit::text() const
 	return result.trimmed();
 }
 
+/******************************************************************************
+* Return the alarm text.
+* If 'showErrorMessage' is true and the text is empty, an error message is
+* displayed.
+*/
+QString CommandEdit::text(EditAlarmDlg* dlg, bool showErrorMessage) const
+{
+	QString result = text();
+	if (showErrorMessage  &&  result.isEmpty())
+		KMessageBox::sorry(dlg, i18nc("@info", "Please enter a command or script to execute"));
+	return result;
+}
+
 /******************************************************************************
 * Set the read-only status of all controls.
 */
