@@ -1,7 +1,7 @@
 /*
  *  messagewin.h  -  displays an alarm message
  *  Program:  kalarm
- *  Copyright © 2001-2008 by David Jarvie <djarvie@kde.org>
+ *  Copyright © 2001-2010 by David Jarvie <djarvie@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -25,22 +25,24 @@
 
 #include <QList>
 #include <QMap>
+#include <QPointer>
 
+#include "autoqpointer.h"
+#include "kaevent.h"
 #include "mainwindowbase.h"
-#include "alarmevent.h"
 
 class QShowEvent;
 class QMoveEvent;
 class QResizeEvent;
 class QCloseEvent;
-class QPushButton;
-class KPushButton;
+class PushButton;
 class MessageText;
 class QCheckBox;
 class QLabel;
 class DeferAlarmDlg;
+class EditAlarmDlg;
 class ShellProcess;
-namespace Phonon { class MediaObject; }
+class AudioThread;
 
 /**
  * MessageWin: A window to display an alarm message
@@ -52,7 +54,8 @@ class MessageWin : public MainWindowBase
 		enum {                // flags for constructor
 			NO_RESCHEDULE = 0x01,    // don't reschedule the event once it has displayed
 			NO_DEFER      = 0x02,    // don't display the Defer button
-			NO_INIT_VIEW  = 0x04     // for internal MessageWin use only
+			ALWAYS_HIDE   = 0x04,    // never show the window (e.g. for audio-only alarms)
+			NO_INIT_VIEW  = 0x08     // for internal MessageWin use only
 		};
 
 		MessageWin();     // for session management restoration only
@@ -62,15 +65,22 @@ class MessageWin : public MainWindowBase
 		void                setRecreating()        { mRecreating = true; }
 		const DateTime&     dateTime()             { return mDateTime; }
 		KAAlarm::Type       alarmType() const      { return mAlarmType; }
-		bool                hasDefer() const       { return !!mDeferButton; }
+		bool                hasDefer() const;
+		void                showDefer();
+		void                cancelReminder(const KAEvent&, const KAAlarm&);
+		void                showDateTime(const KAEvent&, const KAAlarm&);
 		bool                isValid() const        { return !mInvalid; }
+		bool                alwaysHidden() const   { return mAlwaysHide; }
 		virtual void        show();
 		virtual QSize       sizeHint() const;
-		static int          instanceCount()        { return mWindowList.count(); }
+		static int          instanceCount(bool excludeAlwaysHidden = false);
 		static MessageWin*  findEvent(const QString& eventID);
 		static void         redisplayAlarms();
+		static void         stopAudio(bool wait = false);
+		static bool         isAudioPlaying();
 		static void         showError(const KAEvent&, const DateTime& alarmDateTime, const QStringList& errmsgs,
 		                              const QString& dontShowAgain = QString());
+		static bool         spread(bool scatter);
 
 	protected:
 		virtual void        showEvent(QShowEvent*);
@@ -84,26 +94,33 @@ class MessageWin : public MainWindowBase
 		void                slotOk();
 		void                slotEdit();
 		void                slotDefer();
+		void                editCloseOk();
+		void                editCloseCancel();
+		void                activeWindowChanged(WId);
 		void                checkDeferralLimit();
 		void                displayMainWindow();
 #ifdef KMAIL_SUPPORTED
 		void                slotShowKMailMessage();
 #endif
 		void                slotSpeak();
-		void                slotPlayAudio();
-		void                checkAudioPlay();
-		void                stopPlay();
+		void                audioTerminating();
+		void                startAudio();
+		void                playReady();
+		void                playFinished();
 		void                enableButtons();
 		void                setRemainingTextDay();
 		void                setRemainingTextMinute();
-		void                setMaxSize();
+		void                frameDrawn();
 		void                readProcessOutput(ShellProcess*);
 
 	private:
 		MessageWin(const KAEvent*, const DateTime& alarmDateTime, const QStringList& errmsgs,
 		           const QString& dontShowAgain);
 		void                initView();
+		QString             dateTimeToDisplay();
 		void                displayComplete();
+		void                setButtonsReadOnly(bool);
+		bool                getWorkAreaAndModal();
 		void                playAudio();
 		void                setDeferralLimit(const KAEvent&);
 		void                alarmShowing(KAEvent&, const KCal::Event* = 0);
@@ -111,9 +128,13 @@ class MessageWin : public MainWindowBase
 		bool                haveErrorMessage(unsigned msg) const;
 		void                clearErrorMessage(unsigned msg) const;
 		static bool         reinstateFromDisplaying(const KCal::Event*, KAEvent&, AlarmResource*&, bool& showEdit, bool& showDefer);
+		static bool         isSpread(const QPoint& topLeft);
 
 		static QList<MessageWin*> mWindowList;  // list of existing message windows
 		static QMap<QString, unsigned> mErrorMessages;  // error messages currently displayed, by event ID
+		// Sound file playing
+		static QPointer<AudioThread> mAudioThread;     // thread to play audio file
+		static MessageWin*           mAudioOwner;      // window which owns mAudioThread
 		// Properties needed by readProperties()
 		QString             mMessage;
 		QFont               mFont;
@@ -127,8 +148,9 @@ class MessageWin : public MainWindowBase
 		int                 mFadeSeconds;
 		int                 mDefaultDeferMinutes;
 		KAAlarm::Type       mAlarmType;
-		KAEvent::Action     mAction;
+		KAEventData::Action mAction;
 		unsigned long       mKMailSerialNumber; // if email text, message's KMail serial number, else 0
+		KAEvent::CmdErrType mCommandError;
 		QStringList         mErrorMsgs;
 		QString             mDontShowAgain;   // non-null for don't-show-again option with error message
 		int                 mRestoreHeight;
@@ -137,27 +159,29 @@ class MessageWin : public MainWindowBase
 		bool                mShowEdit;        // display the Edit button
 		bool                mNoDefer;         // don't display a Defer option
 		bool                mInvalid;         // restored window is invalid
-		// Sound file playing
-		Phonon::MediaObject* mAudioObject;
-		bool                mPlayedOnce;      // the sound file has started playing at least once
 		// Miscellaneous
 		KAEvent             mEvent;           // the whole event, for updating the calendar file
 		AlarmResource*      mResource;        // resource which the event comes/came from
+		QLabel*             mTimeLabel;       // trigger time label
 		QLabel*             mRemainingText;   // the remaining time (for a reminder window)
-		KPushButton*        mOkButton;
-		QPushButton*        mEditButton;
-		QPushButton*        mDeferButton;
-		QPushButton*        mSilenceButton;
-		QPushButton*        mKAlarmButton;
-		QPushButton*        mKMailButton;
+		PushButton*         mOkButton;
+		PushButton*         mEditButton;
+		PushButton*         mDeferButton;
+		PushButton*         mSilenceButton;
+		PushButton*         mKAlarmButton;
+		PushButton*         mKMailButton;
 		MessageText*        mCommandText;     // shows output from command
 		QCheckBox*          mDontShowAgainCheck;
+		EditAlarmDlg*       mEditDlg;         // alarm edit dialog invoked by Edit button
 		DeferAlarmDlg*      mDeferDlg;
 		QDateTime           mDeferLimit;      // last time to which the message can currently be deferred
-		int                 mFlags;
+		int                 mFlags;           // event flags
 		int                 mLateCancel;
 		int                 mButtonDelay;     // delay (ms) after window is shown before buttons are enabled
+		int                 mScreenNumber;    // screen to display on, or -1 for default
+		bool                mAlwaysHide;      // the window should never be displayed
 		bool                mErrorWindow;     // the window is simply an error message
+		bool                mInitialised;     // initView() has been called to create the window's widgets
 		bool                mNoPostAction;    // don't execute any post-alarm action
 		bool                mRecreating;      // window is about to be deleted and immediately recreated
 		bool                mBeep;
