@@ -41,6 +41,7 @@ using KMail::AccountManager;
 #include "kpimidentities/identity.h"
 #include "kpimidentities/identitymanager.h"
 #include "protocols.h"
+#include "autoqpointer.h"
 
 #include <mailtransport/transport.h>
 #include <mailtransport/transportmanager.h>
@@ -59,6 +60,7 @@ using KMail::IdentityListViewItem;
 #include <kconfiggroup.h>
 #include <kvbox.h>
 #include <kmaccount.h>
+#include <kpushbutton.h>
 
 #include <QCheckBox>
 #include <QDir>
@@ -114,6 +116,9 @@ AccountWizard::AccountWizard( KMKernel *kernel, QWidget *parent )
     mAccount( 0 ), mTransport( 0 ), mServerTest( 0 )
 {
   showButton( Help, false );
+  if ( kmkernel->firstStart() ) {
+    button( KDialog::Cancel )->setText( i18n( "Create an Account Later" ) );
+  }
 
   setupWelcomePage();
   setupAccountTypePage();
@@ -136,8 +141,8 @@ void AccountWizard::start( KMKernel *kernel, QWidget *parent )
 {
   KConfigGroup wizardConfig( KMKernel::config(), "AccountWizard" );
 
-  AccountWizard wizard( kernel, parent );
-  int result = wizard.exec();
+  AutoQPointer<AccountWizard> wizard( new AccountWizard( kernel, parent ) );
+  int result = wizard->exec();
   if ( result == QDialog::Accepted ) {
     wizardConfig.writeEntry( "ShowOnStartup", false );
     kernel->slotConfigChanged();
@@ -478,7 +483,7 @@ void AccountWizard::createTransport()
     if ( pathToSendmail.isEmpty() ) {
       pathToSendmail = KStandardDirs::findExe( "sendmail", "/usr/sbin" );
       if ( pathToSendmail.isEmpty() ) {
-        kWarning(5006) <<"Could not find the sendmail binary for local delivery";
+        kWarning() <<"Could not find the sendmail binary for local delivery";
         // setting the path to the most likely location
         pathToSendmail = "/usr/sbin/sendmail";
       }
