@@ -25,10 +25,14 @@ namespace KMail {
 
   void ISubject::notify()
   {
-    kDebug(5006) << mObserverList.size();
-    for ( QVector<Interface::Observer*>::iterator it = mObserverList.begin() ; it != mObserverList.end() ; ++it )
-      (*it)->update( this );
+    // iterate over a copy (to prevent crashes when
+    // {attach(),detach()} is called from an Observer::update()
+    const QVector<Interface::Observer*> copy = mObserverList;
+    for ( QVector<Interface::Observer*>::const_iterator it = copy.constBegin() ; it != copy.constEnd() ; ++it ) {
+      if ( *it ) {
+        (*it)->update( this );
+      }
+    }
   }
-
 }
 
