@@ -27,7 +27,6 @@ using KMail::SieveConfig;
 #include "kmmessage.h"
 #include "broadcaststatus.h"
 using KPIM::BroadcastStatus;
-#include "kmfoldertree.h"
 #include "kmfoldermgr.h"
 #include "kmfolderimap.h"
 #include "kmmainwin.h"
@@ -70,11 +69,11 @@ KMAcctImap::KMAcctImap(AccountManager* aOwner, const QString& aAccountName, uint
   QString serNumUri = KStandardDirs::locateLocal( "data", "kmail/unfiltered." +
                                             QString("%1").arg(KAccount::id()) );
   KConfig config( serNumUri );
-  QStringList serNums =
+  const QStringList serNums =
       config.group("<default>").readEntry( "unfiltered", QStringList() );
 
-  for ( QStringList::ConstIterator it = serNums.begin();
-	it != serNums.end(); ++it ) {
+  for ( QStringList::ConstIterator it = serNums.constBegin();
+	it != serNums.constEnd(); ++it ) {
       mFilterSerNums.append( (*it).toUInt() );
       mFilterSerNumsToSave.insert( *it, 1 );
     }
@@ -160,7 +159,7 @@ void KMAcctImap::killAllJobs( bool disconnectSlave )
     for ( ; it2 != msgList.end(); ++it2 ) {
        KMMessage *msg = *it2;
        if ( msg->transferInProgress() ) {
-          kDebug(5006) << "Resetting mail";
+          kDebug() << "Resetting mail";
           msg->setTransferInProgress( false );
        }
     }
@@ -180,11 +179,17 @@ void KMAcctImap::killAllJobs( bool disconnectSlave )
       (*it).progressItem->setComplete();
     }
   }
-  if (mSlave && mapJobData.begin() != mapJobData.end())
-  {
-    mSlave->kill();
+
+  if ( slave() ) {
+    if ( disconnectSlave ) {
+      KIO::Scheduler::disconnectSlave( slave() );
+    }
+    if ( mapJobData.begin() != mapJobData.end() ) {
+      mSlave->kill();
+    }
     mSlave = 0;
   }
+
   // remove the jobs
   mapJobData.clear();
   KMAccount::deleteFolderJobs();
@@ -199,10 +204,6 @@ void KMAcctImap::killAllJobs( bool disconnectSlave )
     checkDone( false, CheckOK ); // returned 0 new messages
     mCountRemainChecks = 0;
   }
-  if ( disconnectSlave && slave() ) {
-    KIO::Scheduler::disconnectSlave( slave() );
-    mSlave = 0;
-  }
 }
 
 //-----------------------------------------------------------------------------
@@ -210,10 +211,10 @@ void KMAcctImap::ignoreJobsForMessage( KMMessage* msg )
 {
   if (!msg) return;
   QList<ImapJob*>::const_iterator it;
-  for ( it = mImapJobList.begin(); (*it) && it != mImapJobList.constEnd(); ++it )
+  for ( it = mImapJobList.constBegin(); it != mImapJobList.constEnd() && (*it); ++it )
   {
     ImapJob *job = (*it);
-    if ( job->msgList().first() == msg )
+    if ( !job->msgList().isEmpty() && job->msgList().first() == msg )
     {
       job->kill();
     }
@@ -224,7 +225,7 @@ void KMAcctImap::ignoreJobsForMessage( KMMessage* msg )
 void KMAcctImap::ignoreJobsForFolder( KMFolder* folder )
 {
   QList<ImapJob*>::const_iterator it;
-  for ( it = mImapJobList.begin(); (*it) && it != mImapJobList.constEnd(); ++it )
+  for ( it = mImapJobList.constBegin(); (*it) && it != mImapJobList.constEnd(); ++it )
   {
     ImapJob *job = (*it);
     if ( !job->msgList().isEmpty() && job->msgList().first()->parent() == folder )
@@ -276,7 +277,7 @@ void KMAcctImap::cancelMailCheck()
 //-----------------------------------------------------------------------------
 void KMAcctImap::processNewMail(bool interactive)
 {
-  kDebug(5006) <<"processNewMail" << mCheckingSingleFolder <<",status="<<makeConnection();
+  kDebug() << "processNewMail" << mCheckingSingleFolder <<",status="<<makeConnection();
   if ( !mFolder || !mFolder->folder() || !mFolder->folder()->child() ||
        makeConnection() == ImapAccountBase::Error ) {
     // checks for mCountRemainChecks
@@ -302,7 +303,7 @@ void KMAcctImap::processNewMail(bool interactive)
   mMailCheckProgressItem =
     ProgressManager::createProgressItem(
         "MailCheckAccount" + name(),
-        i18n( "Checking account: %1", Qt::escape( name() ) ),
+        i18nc( "@info:status", "Checking account: %1", Qt::escape( name() ) ),
         QString(), // status
         true, // can be canceled
         useSSL() || useTLS() );
@@ -399,7 +400,8 @@ void KMAcctImap::postProcessNewMail( KMFolder * folder )
   if ( mMailCheckProgressItem ) {
     mMailCheckProgressItem->incCompletedItems();
     mMailCheckProgressItem->updateProgress();
-    mMailCheckProgressItem->setStatus( folder->prettyUrl() + i18n(" completed") );
+    mMailCheckProgressItem->setStatus( folder->prettyUrl()
+      + i18nc("@info:status Number of emails retrieved.", " completed") );
   }
   mCountRemainChecks--;
 
@@ -454,7 +456,7 @@ void KMAcctImap::postProcessNewMail( KMFolder * folder )
     if (idx != -1) {
 
       msg = folder->getMsg( idx );
-      if (!msg) { // sanity checking
+      if ( !msg ) {
         mFilterSerNumsToSave.remove( QString( "%1" ).arg( sernum ) );
         continue;
       }
@@ -506,7 +508,7 @@ void KMAcctImap::slotUpdateFolderList()
 {
   if ( !mFolder || !mFolder->folder() || !mFolder->folder()->child() )
   {
-    kWarning(5006) << "Return";
+    kWarning() << "Return";
     return;
   }
   QStringList strList;
@@ -565,7 +567,7 @@ ImapAccountBase::ConnectionState KMAcctImap::makeConnection()
 void KMAcctImap::slotResetConnectionError()
 {
   mSlaveConnectionError = false;
-  kDebug(5006) ;
+  kDebug() ;
 }
 
 void KMAcctImap::slotFolderSelected( KMFolderImap* folder, bool )
@@ -593,7 +595,16 @@ int KMAcctImap::slotFilterMsg( KMMessage *msg )
     // messageRetrieved(0) is always possible
     return -1;
   }
-  msg->setTransferInProgress(false);
+
+  msg->setTransferInProgress( false );
+
+  if ( msg->parent() && msg->parent()->find( msg ) == -1 ) {
+    // ### Isn't the msg pointer invalid already at this point, because it was removed
+    //     from the message list!? Seems to work, so...
+    kWarning() << "The message with subject" << msg->subject() << "was removed from the server"
+               << "while we were filtering!";
+    return 1; // Yay for clean return enums! (see kmfiltermgr for explaination)
+  }
   quint32 serNum = msg->getMsgSerNum();
   if ( serNum )
     mFilterSerNumsToSave.remove( QString( "%1" ).arg( serNum ) );
@@ -604,7 +615,8 @@ int KMAcctImap::slotFilterMsg( KMMessage *msg )
                                                     id() );
   if (filterResult == 2) {
     // something went horribly wrong (out of space?)
-    kmkernel->emergencyExit( i18n("Unable to process messages: " ) + QString::fromLocal8Bit(strerror(errno)));
+    kmkernel->emergencyExit(
+      i18nc( "@info:status", "Unable to process messages: " ) + QString::fromLocal8Bit(strerror(errno)));
     return 2;
   }
   if (msg->parent()) { // unGet this msg
