@@ -17,6 +17,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  */
+#include <config-kmail.h>
 
 #include "kmfolder.h"
 #include "kmfolderdir.h"
@@ -32,10 +33,16 @@
 #include "compactionjob.h"
 #include "kmailicalifaceimpl.h"
 #include "kmaccount.h"
+#include "globalsettings.h"
 
 #include <errno.h>
 #include <unistd.h> // W_OK
 
+#ifdef INDICATEQT_FOUND
+// libindicate
+#include <qindicateindicator.h>
+#endif
+
 #include <kdebug.h>
 #include <kde_file.h> // KDE_mkdir
 #include <klocale.h>
@@ -61,7 +68,9 @@ KMFolder::KMFolder( KMFolderDir* aParent, const QString& aFolderName,
     mUseCustomIcons( false ), mMailingListEnabled( false ),
     mAcctList( 0 ),
     mPutRepliesInSameFolder( false ),
-    mIgnoreNewMail( false )
+    mHideInSelectionDialog( false ),
+    mIgnoreNewMail( false ),
+    mIndicator( 0 )
 {
   mIdentity = kmkernel->identityManager()->defaultIdentity().uoid();
   if( aFolderType == KMFolderTypeCachedImap )
@@ -214,7 +223,8 @@ void KMFolder::readConfig( KConfigGroup & configGroup )
   if ( savedId != 0 && mId == 0 )
     mId = savedId;
   mPutRepliesInSameFolder = configGroup.readEntry( "PutRepliesInSameFolder", false );
-  mIgnoreNewMail = configGroup.readEntry( "IgnoreNewMail", false );
+  mHideInSelectionDialog = configGroup.readEntry( "HideInSelectionDialog", false );
+  setIgnoreNewMail( configGroup.readEntry( "IgnoreNewMail", false ) );
 
   if ( mUseCustomIcons )
     emit iconsChanged();
@@ -256,6 +266,7 @@ void KMFolder::writeConfig( KConfigGroup & configGroup ) const
   configGroup.writeEntry("WhoField", mUserWhoField);
   configGroup.writeEntry("Id", mId);
   configGroup.writeEntry( "PutRepliesInSameFolder", mPutRepliesInSameFolder );
+  configGroup.writeEntry( "HideInSelectionDialog", mHideInSelectionDialog );
   configGroup.writeEntry( "IgnoreNewMail", mIgnoreNewMail );
   if ( !mShortcut.isEmpty() )
     configGroup.writeEntry( "Shortcut", mShortcut.toString() );
@@ -614,6 +625,11 @@ bool KMFolder::isReadOnly() const
   return mStorage->isReadOnly();
 }
 
+bool KMFolder::isWritable() const
+{
+  return !mStorage->isReadOnly() && mStorage->canDeleteMessages();
+}
+
 bool KMFolder::canDeleteMessages() const
 {
   return mStorage->canDeleteMessages();
@@ -630,6 +646,27 @@ QString KMFolder::label() const
   return name();
 }
 
+void KMFolder::setSystemFolder( bool itIs )
+{
+  mIsSystemFolder = itIs;
+  updateIndicatorText();
+  updateIndicatorIcon();
+}
+
+void KMFolder::setLabel( const QString &l )
+{
+  mLabel = l;
+  updateIndicatorText();
+  updateIndicatorIcon();
+}
+
+void KMFolder::setSystemLabel( const QString &l )
+{
+  mSystemLabel = l;
+  updateIndicatorText();
+  updateIndicatorIcon();
+}
+
 //-----------------------------------------------------------------------------
 QString KMFolder::prettyUrl() const
 {
@@ -735,14 +772,15 @@ void KMFolder::setUserWhoField( const QString& whoField, bool writeConfig )
     mWhoField = whoField;
   else {
     // this should not happen...
-    kDebug() <<"Illegal setting" << whoField << "for userWhoField!";
+    kDebug() << "Illegal setting" << whoField << "for userWhoField!";
     return; // don't use the value
   }
   mUserWhoField = whoField;
 
-  if (writeConfig)
+  if ( writeConfig ) {
     mStorage->writeConfig();
-  emit viewConfigChanged();
+    emit viewConfigChanged();
+  }
 }
 
 void KMFolder::correctUnreadMsgsCount()
@@ -916,6 +954,35 @@ void KMFolder::reallyAddCopyOfMsg( KMMessage* aMsg )
   mStorage->reallyAddCopyOfMsg( aMsg );
 }
 
+void KMFolder::setIgnoreNewMail( bool b )
+{
+  mIgnoreNewMail = b;
+#ifdef INDICATEQT_FOUND
+  if ( b && mIndicator ) {
+    delete mIndicator;
+    mIndicator = 0;
+  } else if ( !b && !mIndicator ) {
+    mIndicator = new QIndicate::Indicator( this );
+    connect( mIndicator, SIGNAL( display( QIndicate::Indicator* ) ),
+      SLOT( slotIndicatorClicked() ) );
+    connect( mStorage, SIGNAL( numUnreadMsgsChanged( KMFolder* ) ),
+      SLOT( updateIndicatorCount() ) );
+    connect( this, SIGNAL( iconsChanged() ),
+      SLOT( updateIndicatorIcon() ) );
+    connect( kmkernel, SIGNAL( configChanged() ),
+      SLOT( updateIndicatorCount() ) );
+
+    // Update components of the indicator, but delay it because it's not good
+    // to ask for a folder count while the application is still starting up
+    // (for example search folder manager in KMKernel might not have been
+    // initialized yet)
+    QMetaObject::invokeMethod( this, "updateIndicatorText", Qt::QueuedConnection );
+    QMetaObject::invokeMethod( this, "updateIndicatorIcon", Qt::QueuedConnection );
+    QMetaObject::invokeMethod( this, "updateIndicatorCount", Qt::QueuedConnection );
+  }
+#endif
+}
+
 void KMFolder::setShortcut( const KShortcut &sc )
 {
   if ( mShortcut != sc ) {
@@ -929,6 +996,86 @@ bool KMFolder::isMoveable() const
   return !isSystemFolder() && mStorage->isMoveable();
 }
 
+void KMFolder::updateIndicatorCount()
+{
+#ifdef INDICATEQT_FOUND
+  if ( !mIndicator ) {
+    return;
+  }
+  if ( !GlobalSettings::useMessageIndicator() ) {
+    mIndicator->hide();
+    return;
+  }
+  int count = mStorage->countUnread();
+  if ( count > 0 ) {
+    mIndicator->setCountProperty( count );
+    mIndicator->setDrawAttentionProperty( true );
+    mIndicator->show();
+  } else {
+    mIndicator->hide();
+  }
+#endif
+}
+
+void KMFolder::updateIndicatorText()
+{
+#ifdef INDICATEQT_FOUND
+  if ( !mIndicator ) {
+    return;
+  }
+  mIndicator->setNameProperty( label() );
+#endif
+}
+
+void KMFolder::updateIndicatorIcon()
+{
+#ifdef INDICATEQT_FOUND
+  if ( !mIndicator ) {
+    return;
+  }
+
+  QString icon;
+  if ( useCustomIcons() ) {
+    icon = unreadIconPath();
+    if ( icon.isEmpty() ) {
+      icon = normalIconPath();
+    }
+  }
+  // Real icon code is in FolderView. This approximation should be good enough.
+  if ( icon.isEmpty() ) {
+    if ( mIsSystemFolder ) {
+      if ( mName == "inbox" ) {
+        icon = "mail-folder-inbox";
+      } else if ( mName == "outbox" ) {
+        icon = "mail-folder-outbox";
+      } else if ( mName == "sent" ) {
+        icon = "mail-folder-sent";
+      } else if ( mName == "trash" ) {
+        icon = "user-trash";
+      } else if ( mName == "drafts" ) {
+        icon = "document-properties";
+      } else if ( mName == "templates" ) {
+        icon = "document-new";
+      }
+    } else {
+      icon = "folder-open";
+    }
+  }
+  QPixmap pix = SmallIcon(icon);
+  if ( pix.isNull() ) {
+    kWarning() << "Could not read image from" << icon;
+  } else {
+    mIndicator->setIconProperty( pix.toImage() );
+  }
+#endif
+}
+
+void KMFolder::slotIndicatorClicked()
+{
+  kmkernel->showMainWin();
+  kmkernel->selectFolder( this );
+}
+
 void KMFolder::slotContentsTypeChanged( KMail::FolderContentsType type )
 {
   kmkernel->iCalIface().folderContentsTypeChanged( this, type );
@@ -960,5 +1107,44 @@ void KMFolder::slotIdentitiesChanged()
   }
 }
 
+bool KMFolder::isValidName( KMFolder * folder, const QString &folderName, QString &message )
+{
+  KMFolderType fldType = folder->folderType();
+
+  // names of local folders must not contain a '/'
+  if ( folderName.contains( "/" )  &&
+       fldType != KMFolderTypeImap &&
+       fldType != KMFolderTypeCachedImap ) {
+    message = i18n( "Folder names cannot contain the / (slash) character; please choose another folder name." );
+    return false;
+  }
+
+  // folder names must not start with a '.'
+  if ( folderName.startsWith( "." ) ) {
+    message = i18n( "Folder names cannot start with a . (dot) character; please choose another folder name." );
+    return false;
+  }
+  FolderStorage *folderStorage = folder->storage();
+  // names of IMAP folders must not contain the folder delimiter
+  if ( fldType == KMFolderTypeImap || fldType == KMFolderTypeCachedImap ) {
+    QString delimiter;
+    if ( fldType == KMFolderTypeImap ) {
+      KMAcctImap *ai = static_cast<KMFolderImap*>( folderStorage )->account();
+      if ( ai ) {
+        delimiter = ai->delimiterForFolder( folderStorage );
+      }
+    } else {
+      KMAcctCachedImap *ai = static_cast<KMFolderCachedImap*>( folderStorage )->account();
+      if ( ai ) {
+        delimiter = ai->delimiterForFolder( folderStorage );
+      }
+    }
+    if ( !delimiter.isEmpty() && folderName.contains( delimiter ) ) {
+      message = i18n( "Your IMAP server does not allow the character '%1'; please choose another folder name.", delimiter );
+      return false;
+    }
+  }
+  return true;
+}
 
 #include "kmfolder.moc"
