@@ -1,26 +1,30 @@
 // -*- mode: C++; c-file-style: "gnu" -*-
 
-#ifndef _KMCONTROL
-#define _KMCONTROL
+#ifndef _KMKERNEL_H
+#define _KMKERNEL_H
 
-#include <qobject.h>
-#include <qstring.h>
-#include <qguardedptr.h>
-#include <weaver.h>
-#include <weaverlogger.h>
+#include <QByteArray>
+#include <QList>
+#include <QObject>
+#include <QString>
+#include <QPointer>
+#include <QDBusObjectPath>
+#include <threadweaver/ThreadWeaver.h>
 
 #include <kconfig.h>
-#include <kdeversion.h>
-#include <kimproxy.h>
-#include <kdepimmacros.h>
+#include <kurl.h>
 
-#include "kmailIface.h"
+#include "kmail_export.h"
 #include "kmmsgbase.h"
+#include "kmmessagetag.h"
 #include "globalsettings.h"
+#include <kcomponentdata.h>
 
 #define kmkernel KMKernel::self()
 #define kmconfig KMKernel::config()
 
+Q_DECLARE_METATYPE(QVector<QStringList>)
+
 namespace KIO {
   class Job;
 }
@@ -28,191 +32,218 @@ namespace KWallet {
   class Wallet;
 }
 
+class KJob;
 /** The KMail namespace contains classes used for KMail.
 * This is to keep them out of the way from all the other
 * un-namespaced classes in libs and the rest of PIM.
 */
 namespace KMail {
   class MailServiceImpl;
+  class MailManagerImpl;
   class UndoStack;
   class JobScheduler;
   class MessageSender;
   class AccountManager;
+  class FolderAdaptor;
 }
 namespace KPIM { class ProgressDialog; }
 using KMail::MailServiceImpl;
+using KMail::MailManagerImpl;
 using KMail::AccountManager;
 using KMail::UndoStack;
 using KMail::JobScheduler;
 using KPIM::ProgressDialog;
-class KMMsgIndex;
-class QLabel;
 class KMFolder;
 class KMFolderMgr;
 class KMFilterMgr;
 class KMFilterActionDict;
 class KMSender;
-namespace KPIM {
+namespace KPIMIdentities {
   class Identity;
   class IdentityManager;
 }
 class KMKernel;
-class KProcess;
-class KProgressDialog;
-class KInstance;
+class KComponentData;
 class QTimer;
-class KProgress;
-class KPassivePopup;
 class KMMainWin;
 class KMainWindow;
 class KMailICalIfaceImpl;
-class KMReaderWin;
-class KSystemTray;
+class KSystemTrayIcon;
 class KMMainWidget;
 class ConfigureDialog;
+class KMMessageTagMgr;
+
+namespace QIndicate {
+  class Server;
+}
 
 /**
  * @short Central point of coordination in KMail
  *
  * The KMKernel class represents the core of KMail, where the different parts
  * come together and are coordinated. It is currently also the class which exports
- * KMail's main DCOP interfaces. The kernel is responsible for creating various
+ * KMail's main D-BUS interfaces. The kernel is responsible for creating various
  * (singleton) objects such as the UndoStack, the folder managers and filter
  * manager, etc.
  */
-class KDE_EXPORT KMKernel : public QObject, virtual public KMailIface
+class KMAIL_EXPORT KMKernel : public QObject
 {
   Q_OBJECT
+  Q_CLASSINFO("D-Bus Interface", "org.kde.kmail.kmail")
 
 public:
-  KMKernel (QObject *parent=0, const char *name=0);
+  explicit KMKernel (QObject *parent=0, const char *name=0);
   ~KMKernel ();
 
-  /** dcop callable stuff */
+/**
+ * Start of D-Bus callable stuff. The D-Bus methods need to be public slots,
+ * otherwise they can't be accessed.
+ */
+public Q_SLOTS:
 
-  void checkMail ();
-  QStringList accounts();
-  void checkAccount (const QString &account);
-  /** returns id of composer if more are opened */
-  int openComposer (const QString &to, const QString &cc, const QString &bcc,
-                    const QString &subject, const QString &body, int hidden,
-                    const KURL &messageFile, const KURL::List &attachURLs,
-                    const QCStringList &customHeaders);
-  /** For backward compatibility */
-  int openComposer (const QString &to, const QString &cc, const QString &bcc,
-                    const QString &subject, const QString &body, int hidden,
-                    const KURL &messageFile, const KURL::List &attachURLs)
-  {
-    QCStringList noCustomHeaders;
-    return openComposer(to, cc, bcc, subject, body, hidden, messageFile, attachURLs, noCustomHeaders);
-  }
-  /** For backward compatibility */
-  int openComposer (const QString &to, const QString &cc, const QString &bcc,
-                    const QString &subject, const QString &body, int hidden,
-                    const KURL &messageFile, const KURL& attachURL)
-  {
-    return openComposer(to, cc, bcc, subject, body, hidden, messageFile, KURL::List(attachURL));
-  }
-  /** For backward compatibility */
-  int openComposer (const QString &to, const QString &cc, const QString &bcc,
-                    const QString &subject, const QString &body, int hidden,
-                    const KURL &messageFile)
-  {
-    return openComposer(to, cc, bcc, subject, body, hidden, messageFile, KURL::List());
-  }
-  /** For backward compatibility
-   * @deprecated
+  Q_SCRIPTABLE void checkMail();
+  Q_SCRIPTABLE void openReader() { openReader( false ); }
+
+  /**
+   * Compact all folders, used for the GUI action (and from D-Bus)
    */
-  int openComposer (const QString &to, const QString &cc,
-                    const QString &bcc, const QString &subject,
-                    const QString &body, int hidden,
-                    const QString &attachName,
-                    const QCString &attachCte,
-                    const QCString &attachData,
-                    const QCString &attachType,
-                    const QCString &attachSubType,
-                    const QCString &attachParamAttr,
-                    const QString &attachParamValue,
-                    const QCString &attachContDisp);
-
-  int openComposer (const QString &to, const QString &cc,
-                    const QString &bcc, const QString &subject,
-                    const QString &body, int hidden,
-                    const QString &attachName,
-                    const QCString &attachCte,
-                    const QCString &attachData,
-                    const QCString &attachType,
-                    const QCString &attachSubType,
-                    const QCString &attachParamAttr,
-                    const QString &attachParamValue,
-                    const QCString &attachContDisp,
-                    const QCString &attachCharset);
-
-  DCOPRef openComposer(const QString &to, const QString &cc,
-                       const QString &bcc, const QString &subject,
-                       const QString &body,bool hidden);
-
-  /** DCOP call used to set the default transport. */
-
-  void setDefaultTransport( const QString & transport );
-
-  /** DCOP call used by the Kontact plugin to create a new message. */
-  DCOPRef newMessage(const QString &to,
-                     const QString &cc,
-                     const QString &bcc,
-                     bool hidden,
-                     bool useFolderId,
-                     const KURL &messageFile,
-                     const KURL &attachURL);
-
-  int sendCertificate( const QString& to, const QByteArray& certData );
-
-  void openReader() { openReader( false ); }
-
-  int dcopAddMessage(const QString & foldername, const QString & messagefile,
-                     const QString & MsgStatusFlags = QString());
-  int dcopAddMessage(const QString & foldername, const KURL & messagefile,
-                     const QString & MsgStatusFlags = QString());
-  void dcopResetAddMessage();
-  /** add messages without rejecting duplicates */
-  int dcopAddMessage_fastImport(const QString & foldername, const QString & messagefile,
-                                const QString & MsgStatusFlags = QString());
-  int dcopAddMessage_fastImport(const QString & foldername, const KURL & messagefile,
-                                const QString & MsgStatusFlags = QString());
-
-  QStringList folderList() const;
-  DCOPRef getFolder( const QString& vpath );
-  void selectFolder( QString folder );
-  int timeOfLastMessageCountChange() const;
-  virtual bool showMail( Q_UINT32 serialNumber, QString messageId );
-  virtual QString getFrom( Q_UINT32 serialNumber );
-  virtual QString debugScheduler();
-  virtual QString debugSernum( Q_UINT32 serialNumber );
-  int viewMessage( const KURL & messageFile );
+  Q_SCRIPTABLE void compactAllFolders();
 
   /**
    * Pauses all background jobs and does not
    * allow new background jobs to be started.
-  */
-  virtual void pauseBackgroundJobs();
+   */
+  Q_SCRIPTABLE void pauseBackgroundJobs();
 
   /**
    * Resumes all background jobs and allows
    * new jobs to be started.
-  */
-  virtual void resumeBackgroundJobs();
+   */
+  Q_SCRIPTABLE void resumeBackgroundJobs();
 
   /**
    * Stops all network related jobs and enter offline mode
    * New network jobs cannot be started.
-  */
-  void stopNetworkJobs();
+   */
+  Q_SCRIPTABLE void stopNetworkJobs();
 
   /**
    * Resumes all network related jobs and enter online mode
    * New network jobs can be started.
-  */
-  void resumeNetworkJobs();
+   */
+  Q_SCRIPTABLE void resumeNetworkJobs();
+
+  Q_SCRIPTABLE QStringList folderList() const;
+
+  Q_SCRIPTABLE QStringList accounts();
+
+  /**
+   * Checks the account with the specified name for new mail.
+   * If the account name is empty, all accounts not excluded from manual
+   * mail check will be checked.
+   */
+  Q_SCRIPTABLE void checkAccount( const QString & account );
+
+  Q_SCRIPTABLE void selectFolder( const QString & folder );
+
+  Q_SCRIPTABLE QString getFolder( const QString & vpath );
+  Q_SCRIPTABLE void showImportArchiveDialog();
+
+  Q_SCRIPTABLE bool canQueryClose();
+
+  /**
+   * D-Bus call used to set the default transport.
+   */
+  Q_SCRIPTABLE void setDefaultTransport( const QString & transport );
+
+  Q_SCRIPTABLE int timeOfLastMessageCountChange() const;
+
+  Q_SCRIPTABLE int dbusAddMessage( const QString & foldername,
+                                   const QString & messageFile,
+                                   const QString & MsgStatusFlags = QString() );
+
+  /**
+   * Add messages without rejecting duplicates
+   */
+  Q_SCRIPTABLE int dbusAddMessage_fastImport( const QString & foldername,
+                                              const QString & messageFile,
+                                              const QString & MsgStatusFlags = QString() );
+
+  Q_SCRIPTABLE void dbusResetAddMessage();
+
+  Q_SCRIPTABLE int sendCertificate( const QString & to,
+                                    const QByteArray & certData );
+
+  Q_SCRIPTABLE bool handleCommandLine( bool noArgsOpensReader );
+
+  Q_SCRIPTABLE QString debugScheduler();
+
+  /**
+   * returns id of composer if more are opened
+   */
+  Q_SCRIPTABLE int openComposer( const QString & to,
+                                 const QString & cc,
+                                 const QString & bcc,
+                                 const QString & subject,
+                                 const QString & body,
+                                 bool hidden,
+                                 const QString & messageFile,
+                                 const QStringList & attachmentPaths,
+                                 const QStringList & customHeaders );
+
+  Q_SCRIPTABLE int openComposer( const QString & to,
+                                 const QString & cc,
+                                 const QString & bcc,
+                                 const QString & subject,
+                                 const QString & body,
+                                 bool hidden,
+                                 const QString & attachName,
+                                 const QByteArray & attachCte,
+                                 const QByteArray  &attachData,
+                                 const QByteArray & attachType,
+                                 const QByteArray & attachSubType,
+                                 const QByteArray & attachParamAttr,
+                                 const QString & attachParamValue,
+                                 const QByteArray & attachContDisp,
+                                 const QByteArray & attachCharset,
+                                 unsigned int identity );
+
+  Q_SCRIPTABLE QDBusObjectPath openComposer( const QString & to,
+                                             const QString & cc,
+                                             const QString & bcc,
+                                             const QString & subject,
+                                             const QString & body,
+                                             bool hidden );
+
+  /**
+   * D-Bus call used by the Kontact plugin to create a new message.
+   */
+  Q_SCRIPTABLE QDBusObjectPath newMessage( const QString & to,
+                                           const QString & cc,
+                                           const QString & bcc,
+                                           bool hidden,
+                                           bool useFolderId,
+                                           const QString & messageFile,
+                                           const QString & attachURL );
+
+  Q_SCRIPTABLE bool showMail( quint32 serialNumber, const QString & messageId );
+
+  Q_SCRIPTABLE QString getFrom( quint32 serialNumber );
+
+  Q_SCRIPTABLE QString debugSernum( quint32 serialNumber );
+
+  Q_SCRIPTABLE int viewMessage( const KUrl & messageFile );
+
+Q_SIGNALS:
+
+  Q_SCRIPTABLE void unreadCountChanged();
+
+/**
+ * End of D-Bus callable stuff
+ */
+
+
+public:
 
   /** A static helper function that asks the user
    * if they want to go online.
@@ -229,15 +260,17 @@ public:
 
   /** normal control stuff */
 
-  static KMKernel *self() { return mySelf; }
-  static KConfig *config();
+  static KMKernel *self();
+  static KSharedConfig::Ptr config();
+  static int storageDebug();
 
   void init();
+  void setupDBus();
   void readConfig();
   void cleanupImapFolders();
   void testDir(const char *_name);
   void recoverDeadLetters();
-  void initFolders(KConfig* cfg);
+  void initFolders(KSharedConfig::Ptr cfg);
   void closeAllKMailWindows();
   void cleanup(void);
   void quit();
@@ -247,21 +280,19 @@ public:
    * method returns.
    */
   bool transferMail( QString & destinationDir );
-  void ungrabPtrKb(void);
-  void kmailMsgHandler(QtMsgType aType, const char* aMsg);
   bool doSessionManagement();
   bool firstInstance() { return the_firstInstance; }
   void setFirstInstance(bool value) { the_firstInstance = value; }
-  void action (bool mailto, bool check, const QString &to, const QString &cc,
+  void action( bool mailto, bool check, const QString &to, const QString &cc,
                const QString &bcc, const QString &subj, const QString &body,
-               const KURL &messageFile, const KURL::List &attach,
-               const QCStringList &customHeaders);
-  void byteArrayToRemoteFile(const QByteArray&, const KURL&,
-			     bool overwrite = FALSE);
-  bool folderIsDraftOrOutbox( const KMFolder * );
-  bool folderIsDrafts( const KMFolder * );
-  bool folderIsTemplates( const KMFolder * );
-  bool folderIsTrash( KMFolder * );
+               const KUrl &messageFile, const KUrl::List &attach,
+               const QStringList &customHeaders );
+  void byteArrayToRemoteFile( const QByteArray&, const KUrl&,
+                              bool overwrite = false );
+  bool folderIsDraftOrOutbox(const KMFolder *);
+  bool folderIsDrafts(const KMFolder *);
+  bool folderIsTemplates(const KMFolder *);
+  bool folderIsTrash(KMFolder *);
   /**
    * Returns true if the folder is one of the sent-mail folders.
    */
@@ -271,8 +302,8 @@ public:
    */
   KMFolder* findFolderById( const QString& idString );
 
-  KInstance *xmlGuiInstance() { return mXmlGuiInstance; }
-  void setXmlGuiInstance( KInstance *instance ) { mXmlGuiInstance = instance; }
+  const KComponentData &xmlGuiInstance() { return mXmlGuiInstance; }
+  void setXmlGuiInstance( const KComponentData &instance ) { mXmlGuiInstance = instance; }
 
   KMFolder *inboxFolder() { return the_inboxFolder; }
   KMFolder *outboxFolder() { return the_outboxFolder; }
@@ -291,26 +322,28 @@ public:
   KMFilterMgr *popFilterMgr() { return the_popFilterMgr; }
   KMFilterActionDict *filterActionDict() { return the_filterActionDict; }
   KMail::MessageSender *msgSender();
-  KMMsgIndex *msgIndex();
+  KMMessageTagMgr *msgTagMgr() { return the_msgTagMgr; }
 
-  KPIM::ThreadWeaver::Weaver *weaver() { return the_weaver; }
+  ThreadWeaver::Weaver *weaver() { return the_weaver; }
   /** return the pointer to the identity manager */
-  KPIM::IdentityManager *identityManager();
+  KPIMIdentities::IdentityManager *identityManager();
 
   JobScheduler* jobScheduler() { return mJobScheduler; }
 
-  /** Compact all folders, used for the gui action (and from DCOP) */
-  void compactAllFolders();
+  QIndicate::Server *indicateServer() { return the_indicateServer; }
+
   /** Expire all folders, used for the gui action */
   void expireAllFoldersNow();
 
   KMailICalIfaceImpl& iCalIface();
 
-  bool firstStart() { return the_firstStart; }
-  QString previousVersion() { return the_previousVersion; }
-  bool startingUp() { return the_startingUp; }
+  bool firstStart() const { return the_firstStart; }
+  /** Mark first start as done */
+  void firstStartDone() { the_firstStart = false; }
+  QString previousVersion() const { return the_previousVersion; }
+  bool startingUp() const { return the_startingUp; }
   void setStartingUp (bool flag) { the_startingUp = flag; }
-  bool shuttingDown() { return the_shuttingDown; }
+  bool shuttingDown() const { return the_shuttingDown; }
   void setShuttingDown(bool flag) { the_shuttingDown = flag; }
   void serverReady (bool flag) { the_server_is_ready = flag; }
 
@@ -325,11 +358,10 @@ public:
    */
   bool haveSystemTrayApplet();
 
-  bool registerSystemTrayApplet( const KSystemTray* );
-  bool unregisterSystemTrayApplet( const KSystemTray* );
+  bool registerSystemTrayApplet( const KSystemTrayIcon* );
+  bool unregisterSystemTrayApplet( const KSystemTrayIcon* );
 
   /// Reimplemented from KMailIface
-  bool handleCommandLine( bool noArgsOpensReader );
   void emergencyExit( const QString& reason );
 
   /** Returns a message serial number that hasn't been used yet. */
@@ -346,12 +378,6 @@ public:
   void setContextMenuShown( bool flag ) { mContextMenuShown = flag; }
   bool contextMenuShown() const { return mContextMenuShown; }
 
-  /**
-   * Get a reference to KMail's KIMProxy instance
-   * @return a pointer to a valid KIMProxy
-   */
-  ::KIMProxy* imProxy();
-
   /**
    * Returns true IFF the user has requested that the current mail checks
    * should be aborted. Needs to be periodically polled.
@@ -368,8 +394,6 @@ public:
    */
   void abortMailCheck();
 
-  bool canQueryClose();
-
   /**
    * Called by the folder tree if the count of unread/total messages changed.
    */
@@ -382,14 +406,22 @@ public:
   KMMainWidget *getKMMainWidget();
 
   /** @return a list of all folders from all folder managers. */
-  QValueList< QGuardedPtr<KMFolder> > allFolders();
+  QList< QPointer<KMFolder> > allFolders();
 
   void raise();
 
-  void loadProfile( const QString& path );
+  /** Custom templates have changed, so all windows using them need
+      to regenerate their menus */
+  void updatedTemplates();
+
+  void selectFolder( KMFolder *folder );
+
+  void updateExpireFolder( const QString& oldFolderName, const QString& newFolderName );
 
-  void saveToProfile( const QString& path ) const;
 public slots:
+  void toggleMainWin();
+  void showMainWin();
+  void hideMainWin();
 
   /// Save contents of all open composer widnows to ~/dead.letter
   void dumpDeadLetters();
@@ -411,16 +443,23 @@ public slots:
 
 protected slots:
   void slotDataReq(KIO::Job*,QByteArray&);
-  void slotResult(KIO::Job*);
+  void slotResult(KJob*);
 
 signals:
   void configChanged();
   void folderRemoved( KMFolder* aFolder );
   void onlineStatusChanged( GlobalSettings::EnumNetworkState::type );
+  void customTemplatesChanged();
+
+private slots:
+  /** Updates identities when a transport has been deleted. */
+  void transportRemoved( int id, const QString &name );
+  /** Updates identities when a transport has been renamed. */
+  void transportRenamed( int id, const QString &oldName, const QString &newName );
 
 private:
+  bool mainWindowIsOnCurrentDesktop();
   void openReader( bool onlyCheck );
-  KMMsgStatus strToStatus(const QString &flags);
   KMFolder *currentFolder();
 
   KMFolder *the_inboxFolder;
@@ -439,12 +478,13 @@ private:
   KMFilterMgr *the_filterMgr;
   KMFilterMgr *the_popFilterMgr;
   KMFilterActionDict *the_filterActionDict;
-  mutable KPIM::IdentityManager *mIdentityManager;
+  QIndicate::Server *the_indicateServer;
+  mutable KPIMIdentities::IdentityManager *mIdentityManager;
   KMSender *the_msgSender;
-  KMMsgIndex *the_msgIndex;
+  KMMessageTagMgr *the_msgTagMgr;
   struct putData
   {
-    KURL url;
+    KUrl url;
     QByteArray data;
     int offset;
   };
@@ -464,10 +504,10 @@ private:
   bool closed_by_user;
   bool the_firstInstance;
   bool mMailCheckAborted;
-  static KMKernel *mySelf;
+
   KSharedConfig::Ptr mConfig;
   QTextCodec *netCodec;
-  KInstance* mXmlGuiInstance;
+  KComponentData mXmlGuiInstance;
   ConfigureDialog *mConfigureDialog;
 
   QTimer *mBackgroundTasksTimer;
@@ -476,9 +516,10 @@ private:
   // temporary mainwin
   KMMainWin *mWin;
   MailServiceImpl *mMailService;
+  MailManagerImpl *mMailManager;
 
   // the time of the last change of the unread or total count of a folder;
-  // this can be queried via DCOP in order to determine whether the counts
+  // this can be queried via D-Bus in order to determine whether the counts
   // need to be updated (e.g. in the Summary in Kontact)
   int mTimeOfLastMessageCountChange;
 
@@ -487,18 +528,26 @@ private:
   // context menus and the pinentry program
   bool mContextMenuShown;
 
-  QValueList<const KSystemTray*> systemTrayApplets;
+  QList<const KSystemTrayIcon*> systemTrayApplets;
 
   /* Weaver */
-  KPIM::ThreadWeaver::Weaver *the_weaver;
-  KPIM::ThreadWeaver::WeaverThreadLogger *the_weaverLogger;
+  ThreadWeaver::Weaver *the_weaver;
 
   KWallet::Wallet *mWallet;
 
-  // variables used by dcopAddMessage()
-  QStringList mAddMessageMsgIds;
-  QString     mAddMessageLastFolder;
-  KMFolder    *mAddMsgCurrentFolder;
+  // variables used by dbusAddMessage()
+  QStringList           mAddMessageMsgIds;
+  QString               mAddMessageLastFolder;
+  KMFolder             *mAddMsgCurrentFolder;
+  KMail::FolderAdaptor *folderAdaptor;
+
+  // special debug area
+  int mStorageDebug;
+
+  // main window management
+  bool mMainWinVisible;
+  QPoint mPosOfMainWin;
+  int mDesktopOfMainWin;
 };
 
-#endif
+#endif // _KMKERNEL_H
