@@ -1,30 +1,30 @@
 // kmmsgpart.cpp
 
-#include <config.h>
-#include <kmimemagic.h>
-#include <kmimetype.h>
-#include <kdebug.h>
-#include <kmdcodec.h>
-
 #include "kmmsgpart.h"
 #include "kmkernel.h"
 #include "kmmessage.h"
 #include "globalsettings.h"
-#include "util.h"
+#include "stringutil.h"
+#include "iconnamecache.h"
 
-#include <kasciistringtools.h>
-#include <kmime_charfreq.h>
-#include <kmime_codecs.h>
+#include <kmime/kmime_charfreq.h>
+#include <kmime/kmime_codecs.h>
 #include <mimelib/enum.h>
 #include <mimelib/utility.h>
 #include <mimelib/string.h>
 
-#include <kiconloader.h>
-#include <qtextcodec.h>
+#include <kmimetype.h>
+#include <kdebug.h>
+#include <kcodecs.h>
+#include <kascii.h>
+
+#include <QTextCodec>
+#include <QList>
 
 #include <assert.h>
 
 using namespace KMime;
+using namespace KMail;
 
 //-----------------------------------------------------------------------------
 KMMessagePart::KMMessagePart()
@@ -37,15 +37,15 @@ KMMessagePart::KMMessagePart()
 KMMessagePart::KMMessagePart( QDataStream & stream )
   : mParent(0), mLoadHeaders(false), mLoadPart(false)
 {
-  unsigned long size;
+  int size;
   stream >> mOriginalContentTypeStr >> mName >> mContentDescription
     >> mContentDisposition >> mCte >> size >> mPartSpecifier;
 
-  KPIM::kAsciiToLower( mContentDisposition.data() );
-  KPIM::kAsciiToUpper( mOriginalContentTypeStr.data() );
+  kAsciiToLower( mContentDisposition.data() );
+  kAsciiToUpper( mOriginalContentTypeStr.data() );
 
   // set the type
-  int sep = mOriginalContentTypeStr.find('/');
+  int sep = mOriginalContentTypeStr.indexOf('/');
   mType = mOriginalContentTypeStr.left(sep);
   mSubtype = mOriginalContentTypeStr.mid(sep+1);
 
@@ -62,19 +62,19 @@ KMMessagePart::~KMMessagePart()
 //-----------------------------------------------------------------------------
 void KMMessagePart::clear()
 {
-  mOriginalContentTypeStr = QCString();
+  mOriginalContentTypeStr = QByteArray();
   mType = "text";
   mSubtype = "plain";
   mCte = "7bit";
-  mContentDescription = QCString();
-  mContentDisposition = QCString();
+  mContentDescription = QByteArray();
+  mContentDisposition = QByteArray();
   mBody.truncate( 0 );
-  mAdditionalCTypeParamStr = QCString();
-  mName = QString::null;
-  mParameterAttribute = QCString();
-  mParameterValue = QString::null;
-  mCharset = QCString();
-  mPartSpecifier = QString::null;
+  mAdditionalCTypeParamStr = QByteArray();
+  mName.clear();
+  mParameterAttribute = QByteArray();
+  mParameterValue.clear();
+  mCharset = QByteArray();
+  mPartSpecifier.clear();
   mBodyDecodedSize = 0;
   mParent = 0;
   mLoadHeaders = false;
@@ -87,12 +87,12 @@ void KMMessagePart::duplicate( const KMMessagePart & msgPart )
 {
   // copy the data of msgPart
   *this = msgPart;
-  // detach the explicitely shared QByteArray
+  // detach the explicitly shared QByteArray
   mBody.detach();
 }
 
 //-----------------------------------------------------------------------------
-int KMMessagePart::decodedSize(void) const
+int KMMessagePart::decodedSize() const
 {
   if (mBodyDecodedSize < 0)
     mBodyDecodedSize = bodyDecodedBinary().size();
@@ -101,28 +101,6 @@ int KMMessagePart::decodedSize(void) const
 
 
 //-----------------------------------------------------------------------------
-void KMMessagePart::setBody(const QCString &aStr)
-{
-  KMail::Util::setFromQCString( mBody, aStr );
-
-  int enc = cte();
-  if (enc == DwMime::kCte7bit || enc == DwMime::kCte8bit || enc == DwMime::kCteBinary)
-    mBodyDecodedSize = mBody.size();
-  else
-    mBodyDecodedSize = -1; // Can't know the decoded size
-}
-
-void KMMessagePart::setBody(const DwString &aStr)
-{
-  mBody.duplicate( aStr.c_str(), aStr.length() );
-
-  int enc = cte();
-  if (enc == DwMime::kCte7bit || enc == DwMime::kCte8bit || enc == DwMime::kCteBinary)
-    mBodyDecodedSize = mBody.size();
-  else
-    mBodyDecodedSize = -1; // Can't know the decoded size
-}
-
 void KMMessagePart::setBody(const QByteArray &aStr)
 {
   mBody = aStr;
@@ -135,12 +113,12 @@ void KMMessagePart::setBody(const QByteArray &aStr)
 }
 
 void KMMessagePart::setBodyFromUnicode( const QString & str ) {
-  QCString encoding = KMMsgBase::autoDetectCharset( charset(), KMMessage::preferredCharsets(), str );
+  QByteArray encoding = KMMsgBase::autoDetectCharset( charset(), KMMessage::preferredCharsets(), str );
   if ( encoding.isEmpty() )
     encoding = "utf-8";
   const QTextCodec * codec = KMMsgBase::codecForName( encoding );
   assert( codec );
-  QValueList<int> dummy;
+  QList<int> dummy;
   setCharset( encoding );
   setBodyAndGuessCte( codec->fromUnicode( str ), dummy, false /* no 8bit */ );
 }
@@ -151,7 +129,7 @@ const QTextCodec * KMMessagePart::codec() const {
   if ( !c ) {
     // Ok, no override and nothing in the message, let's use the fallback
     // the user configured
-    c = KMMsgBase::codecForName( GlobalSettings::self()->fallbackCharacterEncoding().latin1() );
+    c = KMMsgBase::codecForName( GlobalSettings::self()->fallbackCharacterEncoding().toLatin1() );
   }
   if ( !c )
     // no charset means us-ascii (RFC 2045), so using local encoding should
@@ -170,95 +148,37 @@ QString KMMessagePart::bodyToUnicode(const QTextCodec* codec) const {
   return codec->toUnicode( bodyDecoded() );
 }
 
-void KMMessagePart::setCharset( const QCString & c ) {
-  if ( type() != DwMime::kTypeText )
-    kdWarning()
-      << "KMMessagePart::setCharset(): trying to set a charset for a non-textual mimetype." << endl
-      << "Fix this caller:" << endl
-      << "====================================================================" << endl
-      << kdBacktrace( 5 ) << endl
-      << "====================================================================" << endl;
+void KMMessagePart::setCharset( const QByteArray & c ) {
+  if ( type() != DwMime::kTypeText ) {
+    kWarning() << "Trying to set a charset for a non-textual mimetype.";
+  }
   mCharset = c;
 }
 
 //-----------------------------------------------------------------------------
-void KMMessagePart::setBodyEncoded(const QCString& aStr)
+void KMMessagePart::setBodyEncoded(const QByteArray& aStr)
 {
-  mBodyDecodedSize = aStr.size() - 1; // same as aStr.length() but faster - assuming no embedded nuls
-  switch (cte())
-  {
-  case DwMime::kCteQuotedPrintable:
-  case DwMime::kCteBase64:
-    {
-      Codec * codec = Codec::codecForName( cteStr() );
-      assert( codec );
-      // we can't use the convenience function here, since aStr is not
-      // a QByteArray...:
-      mBody.resize( codec->maxEncodedSizeFor( mBodyDecodedSize ) );
-      QCString::ConstIterator iit = aStr.data();
-      QCString::ConstIterator iend = aStr.data() + mBodyDecodedSize;
-      QByteArray::Iterator oit = mBody.begin();
-      QByteArray::ConstIterator oend = mBody.end();
-      if ( !codec->encode( iit, iend, oit, oend ) )
-	kdWarning(5006) << codec->name()
-			<< " codec lies about it's maxEncodedSizeFor( "
-			<< mBodyDecodedSize << " ). Result truncated!" << endl;
-      mBody.truncate( oit - mBody.begin() );
-      break;
-    }
-  default:
-    kdWarning(5006) << "setBodyEncoded: unknown encoding '" << cteStr()
-		    << "'. Assuming binary." << endl;
-    // fall through
-  case DwMime::kCte7bit:
-  case DwMime::kCte8bit:
-  case DwMime::kCteBinary:
-    // This is slow and memory hungry - consider using setBodyEncodedBinary instead!
-    mBody.duplicate( aStr.data(), mBodyDecodedSize );
-    break;
-  }
+  // Qt4: QCString and QByteArray have been merged; this method can be cleaned up
+  setBodyEncodedBinary( aStr );
 }
 
 void KMMessagePart::setBodyAndGuessCte(const QByteArray& aBuf,
-				       QValueList<int> & allowedCte,
-				       bool allow8Bit,
+                                       QList<int> & allowedCte,
+                                       bool allow8Bit,
                                        bool willBeSigned )
 {
-  mBodyDecodedSize = aBuf.size();
-
-  CharFreq cf( aBuf ); // save to pass null arrays...
-
-  allowedCte = KMMessage::determineAllowedCtes( cf, allow8Bit, willBeSigned );
-
-#ifndef NDEBUG
-  DwString dwCte;
-  DwCteEnumToStr(allowedCte[0], dwCte);
-  kdDebug(5006) << "CharFreq returned " << cf.type() << "/"
-	    << cf.printableRatio() << " and I chose "
-	    << dwCte.c_str() << endl;
-#endif
-
-  setCte( allowedCte[0] ); // choose best fitting
-  setBodyEncodedBinary( aBuf );
-}
-
-void KMMessagePart::setBodyAndGuessCte(const QCString& aBuf,
-				       QValueList<int> & allowedCte,
-				       bool allow8Bit,
-                                       bool willBeSigned )
-{
-  mBodyDecodedSize = aBuf.size() - 1; // same as aStr.length() but faster - assuming no embedded nuls
+  mBodyDecodedSize = aBuf.length();
 
   CharFreq cf( aBuf.data(), mBodyDecodedSize ); // save to pass null strings
 
-  allowedCte = KMMessage::determineAllowedCtes( cf, allow8Bit, willBeSigned );
+  allowedCte = StringUtil::determineAllowedCtes( cf, allow8Bit, willBeSigned );
 
 #ifndef NDEBUG
   DwString dwCte;
   DwCteEnumToStr(allowedCte[0], dwCte);
-  kdDebug(5006) << "CharFreq returned " << cf.type() << "/"
-	    << cf.printableRatio() << " and I chose "
-	    << dwCte.c_str() << endl;
+  kDebug() << "CharFreq returned" << cf.type() <<"/"
+      << cf.printableRatio() << "and I chose"
+      << dwCte.c_str();
 #endif
 
   setCte( allowedCte[0] ); // choose best fitting
@@ -284,24 +204,25 @@ void KMMessagePart::setBodyEncodedBinary(const QByteArray& aStr)
       assert( codec );
       // Nice: We can use the convenience function :-)
       mBody = codec->encode( aStr );
+      // QP encoding does CRLF -> LF conversion, which can change the size after decoding again
+      // and a size mismatch triggers an assert in various other methods
+      mBodyDecodedSize = -1;
       break;
     }
   default:
-    kdWarning(5006) << "setBodyEncodedBinary: unknown encoding '" << cteStr()
-		    << "'. Assuming binary." << endl;
+    kWarning() <<"setBodyEncodedBinary: unknown encoding '" << cteStr()
+        << "'. Assuming binary.";
     // fall through
   case DwMime::kCte7bit:
   case DwMime::kCte8bit:
   case DwMime::kCteBinary:
-    //mBody.duplicate( aStr );
     mBody = aStr;
-    // Caller has to detach before it modifies aStr!
     break;
   }
 }
 
 //-----------------------------------------------------------------------------
-void KMMessagePart::setMessageBody( const QByteArray& aBuf )
+void KMMessagePart::setMessageBody( const QByteArray &aBuf )
 {
   CharFreq cf( aBuf ); // it's safe to pass null arrays
   mBodyDecodedSize = aBuf.size();
@@ -310,16 +231,17 @@ void KMMessagePart::setMessageBody( const QByteArray& aBuf )
   switch ( cf.type() ) {
   case CharFreq::SevenBitText:
   case CharFreq::SevenBitData:
-     cte = DwMime::kCte7bit;
-     break;
+    cte = DwMime::kCte7bit;
+    break;
   case CharFreq::EightBitText:
   case CharFreq::EightBitData:
-     cte = DwMime::kCte8bit;
-     break;
+    cte = DwMime::kCte8bit;
+    break;
   default:
-     kdWarning(5006) << "Calling " << k_funcinfo
-         << " with something containing neither 7 nor 8 bit text!"
-         << " Fix this caller: " << kdBacktrace() << endl;
+    kWarning() <<"Calling"
+                   << "with something containing neither 7 nor 8 bit text!"
+                   << "Fix this caller:" << kBacktrace();
+    cte = 0;
   }
   setCte( cte );
   setBodyEncodedBinary( aBuf );
@@ -336,80 +258,34 @@ QByteArray KMMessagePart::bodyDecodedBinary() const
     case DwMime::kCte7bit:
     case DwMime::kCte8bit:
     case DwMime::kCteBinary:
-      result.duplicate(mBody);
+      result = mBody;
       break;
     default:
       if ( const Codec * codec = Codec::codecForName( cteStr() ) )
         // Nice: we can use the convenience function :-)
         result = codec->decode( mBody );
       else {
-        kdWarning(5006) << "bodyDecodedBinary: unknown encoding '" << cteStr()
-                        << "'. Assuming binary." << endl;
-	result.duplicate(mBody);
+        kWarning() <<"bodyDecodedBinary: unknown encoding '" << cteStr()
+                        << "'. Assuming binary.";
+        result = mBody;
       }
   }
 
-  assert( mBodyDecodedSize < 0
-	  || (unsigned int)mBodyDecodedSize == result.size() );
+  assert( mBodyDecodedSize < 0 || mBodyDecodedSize == result.size() );
   if ( mBodyDecodedSize < 0 )
     mBodyDecodedSize = result.size(); // cache the decoded size.
 
   return result;
 }
 
-QCString KMMessagePart::bodyDecoded(void) const
+QByteArray KMMessagePart::bodyDecoded(void) const
 {
-  if (mBody.isEmpty()) return QCString("");
-  bool decodeBinary = false;
-  QCString result;
-  int len;
+  if (mBody.isEmpty()) return QByteArray("");
 
-  switch (cte())
-  {
-    case DwMime::kCte7bit:
-    case DwMime::kCte8bit:
-    case DwMime::kCteBinary:
-    {
-      decodeBinary = true;
-      break;
-    }
-    default:
-      if ( const Codec * codec = Codec::codecForName( cteStr() ) ) {
-        // We can't use the codec convenience functions, since we must
-        // return a QCString, not a QByteArray:
-        int bufSize = codec->maxDecodedSizeFor( mBody.size() ) + 1; // trailing NUL
-        result.resize( bufSize );
-        QByteArray::ConstIterator iit = mBody.begin();
-        QCString::Iterator oit = result.begin();
-        QCString::ConstIterator oend = result.begin() + bufSize;
-        if ( !codec->decode( iit, mBody.end(), oit, oend ) )
-          kdWarning(5006) << codec->name()
-                          << " lies about it's maxDecodedSizeFor( "
-                          << mBody.size() << " ). Result truncated!" << endl;
-        len = oit - result.begin();
-        result.truncate( len ); // adds trailing NUL
-      } else {
-        kdWarning(5006) << "bodyDecoded: unknown encoding '" << cteStr()
-                        << "'. Assuming binary." << endl;
-        decodeBinary = true;
-      }
-  }
-
-  if ( decodeBinary ) {
-    len = mBody.size();
-    KMail::Util::setFromByteArray( result, mBody );
-  }
-
-  // Calls length -> slow
-  //kdWarning( result.length() != (unsigned int)len, 5006 )
-  //  << "KMMessagePart::bodyDecoded(): body is binary but used as text!" << endl;
+  QByteArray result = bodyDecodedBinary();
 
   result = result.replace( "\r\n", "\n" ); // CRLF -> LF conversion
 
-  assert( mBodyDecodedSize < 0 || mBodyDecodedSize == len );
-  if ( mBodyDecodedSize < 0 )
-    mBodyDecodedSize = len; // cache decoded size
-
   return result;
 }
 
@@ -417,42 +293,42 @@ QCString KMMessagePart::bodyDecoded(void) const
 //-----------------------------------------------------------------------------
 void KMMessagePart::magicSetType(bool aAutoDecode)
 {
-  KMimeMagic::self()->setFollowLinks( true ); // is it necessary ?
-
   const QByteArray body = ( aAutoDecode ) ? bodyDecodedBinary() : mBody ;
-  KMimeMagicResult * result = KMimeMagic::self()->findBufferType( body );
+  KMimeType::Ptr mime = KMimeType::findByContent( body );
 
-  QString mimetype = result->mimeType();
-  const int sep = mimetype.find('/');
-  mType = mimetype.left(sep).latin1();
-  mSubtype = mimetype.mid(sep+1).latin1();
+  QString mimetype = mime->name();
+  const int sep = mimetype.indexOf('/');
+  mType = mimetype.left(sep).toLatin1();
+  mSubtype = mimetype.mid(sep+1).toLatin1();
 }
 
-
 //-----------------------------------------------------------------------------
-QString KMMessagePart::iconName() const
+QString KMMessagePart::iconName( int size ) const
 {
-  QCString mimeType( mType + "/" + mSubtype );
-  KPIM::kAsciiToLower( mimeType.data() );
+  QByteArray mimeType( mType + '/' + mSubtype );
+  kAsciiToLower( mimeType.data() );
+
+  QString fileName;
+  KMimeType::Ptr mime = KMimeType::mimeType( mimeType, KMimeType::ResolveAliases );
+  if (mime) {
+    fileName = mime->iconName();
+  } else {
+    kWarning() <<"unknown mimetype" << mimeType;
+  }
 
-  QString fileName =
-    KMimeType::mimeType( mimeType )->icon( QString::null, false );
   if ( fileName.isEmpty() )
   {
     fileName = this->fileName();
     if ( fileName.isEmpty() ) fileName = this->name();
     if ( !fileName.isEmpty() )
     {
-      fileName = KMimeType::findByPath( "/tmp/"+fileName, 0, true )->icon( QString::null, true );
+      fileName = KMimeType::findByPath( "/tmp/"+fileName, 0, true )->iconName();
     }
   }
 
-  fileName =
-    KGlobal::instance()->iconLoader()->iconPath( fileName, KIcon::Desktop );
-  return fileName;
+  return IconNameCache::instance()->iconPath( fileName, size );
 }
 
-
 //-----------------------------------------------------------------------------
 int KMMessagePart::type() const {
   return DwTypeStrToEnum(DwString(mType));
@@ -482,7 +358,7 @@ void KMMessagePart::setSubtype(int aSubtype)
 }
 
 //-----------------------------------------------------------------------------
-QCString KMMessagePart::parameterAttribute(void) const
+QByteArray KMMessagePart::parameterAttribute(void) const
 {
   return mParameterAttribute;
 }
@@ -494,7 +370,7 @@ QString KMMessagePart::parameterValue(void) const
 }
 
 //-----------------------------------------------------------------------------
-void KMMessagePart::setParameter(const QCString &attribute,
+void KMMessagePart::setParameter(const QByteArray &attribute,
                                  const QString &value)
 {
   mParameterAttribute = attribute;
@@ -502,7 +378,7 @@ void KMMessagePart::setParameter(const QCString &attribute,
 }
 
 //-----------------------------------------------------------------------------
-QCString KMMessagePart::contentTransferEncodingStr(void) const
+QByteArray KMMessagePart::contentTransferEncodingStr(void) const
 {
   return mCte;
 }
@@ -516,7 +392,7 @@ int KMMessagePart::contentTransferEncoding(void) const
 
 
 //-----------------------------------------------------------------------------
-void KMMessagePart::setContentTransferEncodingStr(const QCString &aStr)
+void KMMessagePart::setContentTransferEncodingStr(const QByteArray &aStr)
 {
     mCte = aStr;
 }
@@ -533,71 +409,71 @@ void KMMessagePart::setContentTransferEncoding(int aCte)
 
 
 //-----------------------------------------------------------------------------
-QString KMMessagePart::contentDescription(void) const
+QString KMMessagePart::contentDescription( void ) const
 {
-  return KMMsgBase::decodeRFC2047String(mContentDescription, charset());
+  return KMMsgBase::decodeRFC2047String( mContentDescription, charset() );;
 }
 
 
 //-----------------------------------------------------------------------------
-void KMMessagePart::setContentDescription(const QString &aStr)
+void KMMessagePart::setContentDescription( const QString &aStr )
 {
-  QCString encoding = KMMsgBase::autoDetectCharset(charset(),
-    KMMessage::preferredCharsets(), aStr);
-  if (encoding.isEmpty()) encoding = "utf-8";
-  mContentDescription = KMMsgBase::encodeRFC2047String(aStr, encoding);
+  QByteArray encoding =
+    KMMsgBase::autoDetectCharset( charset(),
+                                  KMMessage::preferredCharsets(), aStr );
+  if ( encoding.isEmpty() ) {
+    encoding = "utf-8";
+  }
+  mContentDescription = KMMsgBase::encodeRFC2047String( aStr, encoding );
 }
 
 
 //-----------------------------------------------------------------------------
-QString KMMessagePart::fileName(void) const
+QString KMMessagePart::fileName( void ) const
 {
-  QCString str;
-
-  // Allow for multiple filname*0, filename*1, ... params (defined by RFC 2231)
-  // in the Content-Disposision
-  if ( mContentDisposition.contains( "filename*", FALSE ) ) {
-
-    // It's RFC 2231 encoded, so extract the file name with the 2231 method
-    str = KMMsgBase::extractRFC2231HeaderField( mContentDisposition, "filename" );
-    return KMMsgBase::decodeRFC2231String(str);
+  bool bRFC2231encoded = false;
 
+  // search the start of the filename
+  QString cd( mContentDisposition );
+  int startOfFilename = cd.indexOf( "filename*=" );
+  if ( startOfFilename >= 0 ) {
+    bRFC2231encoded = true;
+    startOfFilename += 10;
   } else {
-
-    // Standard RFC 2047-encoded
-    // search the start of the filename
-    int startOfFilename = mContentDisposition.find("filename=", 0, FALSE);
-    if (startOfFilename < 0)
-      return QString::null;
+    startOfFilename = cd.indexOf( "filename=" );
+    if ( startOfFilename < 0 ) {
+      return QString();
+    }
     startOfFilename += 9;
-
-    // search the end of the filename
-    int endOfFilename;
-    if ( '"' == mContentDisposition[startOfFilename] ) {
-      startOfFilename++; // the double quote isn't part of the filename
-      endOfFilename = mContentDisposition.find('"', startOfFilename) - 1;
-    }
-    else {
-      endOfFilename = mContentDisposition.find(';', startOfFilename) - 1;
-    }
-    if (endOfFilename < 0)
-      endOfFilename = 32767;
-
-    const QCString str = mContentDisposition.mid(startOfFilename,
-                                endOfFilename-startOfFilename+1)
-                           .stripWhiteSpace();
-    return KMMsgBase::decodeRFC2047String(str, charset());
   }
 
-  return QString::null;
+  // search the end of the filename
+  int endOfFilename;
+  if ( '"' == mContentDisposition[startOfFilename] ) {
+    startOfFilename++; // the double quote isn't part of the filename
+    endOfFilename = mContentDisposition.indexOf( '"', startOfFilename ) - 1;
+  } else {
+    endOfFilename = mContentDisposition.indexOf( ';', startOfFilename ) - 1;
+  }
+  if ( endOfFilename < 0 ) {
+    endOfFilename = 32767;
+  }
+
+  const QByteArray str =
+    mContentDisposition.mid( startOfFilename,
+                             endOfFilename-startOfFilename + 1 ).trimmed();
+
+  if ( bRFC2231encoded ) {
+    return KMMsgBase::decodeRFC2231String( str );
+  } else {
+    return KMMsgBase::decodeRFC2047String( str, charset() );
+  }
 }
 
-QCString KMMessagePart::body() const
+
+
+QByteArray KMMessagePart::body() const
 {
-  return QCString( mBody.data(), mBody.size() + 1 ); // space for trailing NUL
+  return mBody;
 }
 
-DwString KMMessagePart::dwBody() const
-{
-  return KMail::Util::dwString( mBody );
-}
