@@ -35,17 +35,20 @@
 
 #include "kmmsgbase.h"
 
-#include <qcstring.h>
-
 #include <kleo/cryptobackend.h>
-#include <gpgmepp/verificationresult.h>
+#include <gpgme++/verificationresult.h>
+
+#include <cassert>
 
 class KMReaderWin;
 class KMMessagePart;
 class QString;
-class QWidget;
 class partNode;
 
+namespace GpgME {
+  class Error;
+}
+
 namespace KMail {
 
   class AttachmentStrategy;
@@ -55,10 +58,10 @@ namespace KMail {
 
   class ProcessResult {
   public:
-    ProcessResult( KMMsgSignatureState  inlineSignatureState  = KMMsgNotSigned,
-                   KMMsgEncryptionState inlineEncryptionState = KMMsgNotEncrypted,
-                   bool neverDisplayInline = false,
-                   bool isImage = false )
+    explicit ProcessResult( KMMsgSignatureState  inlineSignatureState  = KMMsgNotSigned,
+            		    KMMsgEncryptionState inlineEncryptionState = KMMsgNotEncrypted,
+    			    bool neverDisplayInline = false,
+    			    bool isImage = false )
       : mInlineSignatureState( inlineSignatureState ),
         mInlineEncryptionState( inlineEncryptionState ),
         mNeverDisplayInline( neverDisplayInline ),
@@ -102,7 +105,7 @@ namespace KMail {
     /** Internal. Copies the context of @p other, but not it's rawReplyString() */
     ObjectTreeParser( const ObjectTreeParser & other );
   public:
-    ObjectTreeParser( KMReaderWin * reader=0, const Kleo::CryptoBackend::Protocol * protocol=0,
+    explicit ObjectTreeParser( KMReaderWin * reader=0, const Kleo::CryptoBackend::Protocol * protocol=0,
                       bool showOneMimePart=false, bool keepEncryptions=false,
                       bool includeSignatures=true,
                       const KMail::AttachmentStrategy * attachmentStrategy=0,
@@ -110,13 +113,20 @@ namespace KMail {
                       KMail::CSSHelper * cssHelper=0 );
     virtual ~ObjectTreeParser();
 
-    QCString rawReplyString() const { return mRawReplyString; }
+    void setAllowAsync( bool allow ) { assert( !mHasPendingAsyncJobs ); mAllowAsync = allow; }
+
+    // ### disabled async mode because of crashes, see bug 208353
+    bool allowAsync() const { return /*mAllowAsync*/false; }
+
+    bool hasPendingAsyncJobs() const { return mHasPendingAsyncJobs; }
+
+    QByteArray rawReplyString() const { return mRawReplyString; }
 
     /*! @return the text of the message, ie. what would appear in the
         composer's text editor if this was edited. */
     QString textualContent() const { return mTextualContent; }
 
-    QCString textualContentCharset() const { return mTextualContentCharset; }
+    QByteArray textualContentCharset() const { return mTextualContentCharset; }
 
     void setCryptoProtocol( const Kleo::CryptoBackend::Protocol * protocol ) {
       mCryptoProtocol = protocol;
@@ -161,16 +171,24 @@ namespace KMail {
 
     void defaultHandling( partNode * node, ProcessResult & result );
 
-    /** 1. Create a new partNode using 'content' data and Content-Description
-            found in 'cntDesc'.
-        2. Make this node the child of 'node'.
-        3. Insert the respective entries in the Mime Tree Viewer.
-        3. Parse the 'node' to display the content. */
+    /**
+     *  1. Create a new partNode using 'content' data and Content-Description
+     *      found in 'cntDesc'.
+     *  2. Make this node the child of 'node'.
+     *  3. Insert the respective entries in the Mime Tree Viewer.
+     *  3. Parse the 'node' to display the content.
+     *
+     * @param addToTextualContent If true, this will add the textual content of the parsed node
+     *                            to the textual content of the current object tree parser.
+     *                            Setting this to false is useful for encapsulated messages, as we
+     *                            do not want the text in those to appear in the editor
+     */
     //  Function will be replaced once KMime is alive.
     void insertAndParseNewChildNode( partNode & node,
                                      const char * content,
                                      const char * cntDesc,
-                                     bool append=false );
+                                     bool append=false,
+                                     bool addToTextualContent = true );
     /** if data is 0:
         Feeds the HTML widget with the contents of the opaque signed
             data found in partNode 'sign'.
@@ -185,20 +203,30 @@ namespace KMail {
                                            partNode & sign,
                                            const QString & fromAddress,
                                            bool doCheck=true,
-                                           QCString * cleartextData=0,
-                                           std::vector<GpgME::Signature> paramSignatures = std::vector<GpgME::Signature>(),
+                                           QByteArray * cleartextData=0,
+                                           const std::vector<GpgME::Signature> & paramSignatures = std::vector<GpgME::Signature>(),
                                            bool hideErrors=false );
 
+    /** Writes out the block that we use when the node is encrypted,
+        but we're deferring decryption for later. */
+    void writeDeferredDecryptionBlock();
+
+    /** Writes out the block that we use when the node is encrypted,
+        but we've just kicked off async decryption. */
+    void writeDecryptionInProgressBlock();
+
     /** Returns the contents of the given multipart/encrypted
         object. Data is decypted.  May contain body parts. */
     bool okDecryptMIME( partNode& data,
-                        QCString& decryptedData,
+                        QByteArray& decryptedData,
                         bool& signatureFound,
                         std::vector<GpgME::Signature> &signatures,
                         bool showWarning,
                         bool& passphraseError,
                         bool& actuallyEncrypted,
+                        bool& decryptionStarted,
                         QString& aErrorText,
+                        GpgME::Error & auditLogError,
                         QString& auditLog );
 
     bool processMailmanMessage( partNode * node );
@@ -207,7 +235,7 @@ namespace KMail {
         we only check whether @p str contains 'xxx="http[s]:' where xxx is
         not href. Obfuscated external references are ignored on purpose.
     */
-    static bool containsExternalReferences( const QCString & str );
+    static bool containsExternalReferences( const QString & str );
 
   public:// (during refactoring)
 
@@ -230,12 +258,12 @@ namespace KMail {
 
   private:
     bool decryptChiasmus( const QByteArray& data, QByteArray& bodyDecoded, QString& errorText );
-    void writeBodyString( const QCString & bodyString,
+    void writeBodyString( const QByteArray & bodyString,
                           const QString & fromAddress,
                           const QTextCodec * codec,
                           ProcessResult & result, bool decorate );
 
-    void writePartIcon( KMMessagePart * msgPart, int partNumber, bool inlineImage=false );
+    void writePartIcon( KMMessagePart * msgPart, int partNumber, bool inlineImage = false );
 
     QString sigStatusToString( const Kleo::CryptoBackend::Protocol * cryptProto,
                                int status_code,
@@ -245,17 +273,23 @@ namespace KMail {
     QString writeSigstatHeader( KMail::PartMetaData & part,
                                 const Kleo::CryptoBackend::Protocol * cryptProto,
                                 const QString & fromAddress,
-                                const QString & filename = QString::null );
+                                partNode *node = 0 );
     QString writeSigstatFooter( KMail::PartMetaData & part );
 
-    void writeBodyStr( const QCString & bodyString,
+    // The attachment mark is a div that is placed around the attchment. It is used for drawing
+    // a yellow border around the attachment when scrolling to it. When scrolling to it, the border
+    // color of the div is changed, see KMReaderWin::scrollToAttachment().
+    void writeAttachmentMarkHeader( partNode *node );
+    void writeAttachmentMarkFooter();
+
+    void writeBodyStr( const QByteArray & bodyString,
                        const QTextCodec * aCodec,
                        const QString & fromAddress,
                        KMMsgSignatureState &  inlineSignatureState,
                        KMMsgEncryptionState & inlineEncryptionState,
                        bool decorate );
   public: // KMReaderWin still needs this...
-    void writeBodyStr( const QCString & bodyString,
+    void writeBodyStr( const QByteArray & bodyString,
                        const QTextCodec * aCodec,
                        const QString & fromAddress );
 
@@ -265,6 +299,11 @@ namespace KMail {
     QString quotedHTML(const QString& pos, bool decorate);
 
     const QTextCodec * codecFor( partNode * node ) const;
+    /** Check if the newline at position @p newLinePos in string @p s
+        seems to separate two paragraphs (important for correct BiDi
+        behavior, but is heuristic because paragraphs are not
+        well-defined) */
+    bool looksLikeParaBreak(const QString& s, unsigned int newLinePos) const;
 
 #ifdef MARCS_DEBUG
     void dumpToFile( const char * filename, const char * dataStart, size_t dataLen );
@@ -274,13 +313,27 @@ namespace KMail {
 
   private:
     KMReaderWin * mReader;
-    QCString mRawReplyString;
-    QCString mTextualContentCharset;
+    QByteArray mRawReplyString;
+    QByteArray mTextualContentCharset;
     QString mTextualContent;
     const Kleo::CryptoBackend::Protocol * mCryptoProtocol;
+
+    /// Show only one mime part means that the user has selected some node in the message structure
+    /// viewer that is not the root, which means the user wants to only see the selected node and its
+    /// children. If that is the case, this variable is set to true.
+    /// The code needs to behave differently if this is set. For example, it should not process the
+    /// siblings. Also, consider inline images: Normally, those nodes are completely hidden, as the
+    /// HTML node embedds them. However, when showing only the node of the image, one has to show them,
+    /// as their is no HTML node in which they are displayed. There are many more cases where this
+    /// variable needs to be obeyed.
+    /// This variable is set to false again when processing the children in stdChildHandling(), as
+    /// the children can be completely displayed again.
     bool mShowOnlyOneMimePart;
+
     bool mKeepEncryptions;
     bool mIncludeSignatures;
+    bool mHasPendingAsyncJobs;
+    bool mAllowAsync;
     const KMail::AttachmentStrategy * mAttachmentStrategy;
     KMail::HtmlWriter * mHtmlWriter;
     KMail::CSSHelper * mCSSHelper;
