@@ -12,12 +12,10 @@
     Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, US
 */
 
-#include <stdlib.h>
-
-#include <qdir.h>
+#include <QDir>
 
 #include <kdebug.h>
-#include <ksimpleconfig.h>
+#include <kconfig.h>
 #include <klocale.h>
 #include <kmessagebox.h>
 #include <kstandarddirs.h>
@@ -34,28 +32,18 @@
 KWallet::Wallet* KNAccountManager::mWallet = 0;
 bool KNAccountManager::mWalletOpenFailed = false;
 
-KNAccountManager::KNAccountManager(KNGroupManager *gm, QObject * parent, const char * name)
-  : QObject(parent, name), gManager(gm), c_urrentAccount(0),
+KNAccountManager::KNAccountManager( KNGroupManager *gm, QObject * parent )
+  : QObject( parent ), gManager( gm ), c_urrentAccount( 0 ),
   mAsyncOpening( false )
 {
-  s_mtp = new KNServerInfo();
-  s_mtp->setType(KNServerInfo::STsmtp);
-  s_mtp->setId(0);
-  KConfig *conf = knGlobals.config();
-  conf->setGroup("MAILSERVER");
-  s_mtp->readConf(conf);
-
   loadAccounts();
 }
 
 
 KNAccountManager::~KNAccountManager()
 {
-  QValueList<KNNntpAccount*>::Iterator it;
-  for ( it = mAccounts.begin(); it != mAccounts.end(); ++it )
-    delete (*it);
+  qDeleteAll( mAccounts );
   mAccounts.clear();
-  delete s_mtp;
   delete mWallet;
   mWallet = 0;
 }
@@ -63,22 +51,21 @@ KNAccountManager::~KNAccountManager()
 
 void KNAccountManager::prepareShutdown()
 {
-  QValueList<KNNntpAccount*>::Iterator it;
-  for ( it = mAccounts.begin(); it != mAccounts.end(); ++it )
+  for ( List::Iterator it = mAccounts.begin(); it != mAccounts.end(); ++it )
     (*it)->saveInfo();
 }
 
 
 void KNAccountManager::loadAccounts()
 {
-  QString dir(locateLocal("data","knode/"));
+  QString dir( KStandardDirs::locateLocal( "data", "knode/" ) );
   if (dir.isNull()) {
     KNHelper::displayInternalFileError();
     return;
   }
   QDir d(dir);
   KNNntpAccount *a;
-  QStringList entries(d.entryList("nntp.*", QDir::Dirs));
+  QStringList entries(d.entryList(QStringList("nntp.*"), QDir::Dirs));
 
   QStringList::Iterator it;
   for(it = entries.begin(); it != entries.end(); ++it) {
@@ -89,7 +76,7 @@ void KNAccountManager::loadAccounts()
       emit accountAdded(a);
     } else {
       delete a;
-      kdError(5003) << "Unable to load account " << (*it) << "!" << endl;
+      kError(5003) <<"Unable to load account" << (*it) <<"!";
     }
   }
 }
@@ -99,8 +86,7 @@ KNNntpAccount* KNAccountManager::account( int id )
 {
   if ( id <= 0 )
     return 0;
-  QValueList<KNNntpAccount*>::ConstIterator it;
-  for ( it = mAccounts.begin(); it != mAccounts.end(); ++it )
+  for ( List::ConstIterator it = mAccounts.constBegin(); it != mAccounts.constEnd(); ++it )
     if ( (*it)->id() == id )
       return *it;
   return 0;
@@ -117,22 +103,22 @@ void KNAccountManager::setCurrentAccount(KNNntpAccount *a)
 bool KNAccountManager::newAccount(KNNntpAccount *a)
 {
   // find a unused id for the new account...
-  QString dir(locateLocal("data","knode/"));
+  QString dir( KStandardDirs::locateLocal( "data", "knode/" ) );
   if (dir.isNull()) {
     delete a;
     KNHelper::displayInternalFileError();
     return false;
   }
   QDir d(dir);
-  QStringList entries(d.entryList("nntp.*", QDir::Dirs));
+  QStringList entries = d.entryList( QStringList( "nntp.*" ), QDir::Dirs );
 
   int id = 1;
-  while (entries.findIndex(QString("nntp.%1").arg(id))!=-1)
+  while (entries.indexOf(QString("nntp.%1").arg(id))!=-1)
     ++id;
 
   a->setId(id);
 
-  dir = locateLocal("data",QString("knode/nntp.%1/").arg(a->id()));
+  dir = KStandardDirs::locateLocal( "data", QString( "knode/nntp.%1/" ).arg( a->id() ) );
   if (!dir.isNull()) {
     mAccounts.append(a);
     emit(accountAdded(a));
@@ -151,30 +137,31 @@ bool KNAccountManager::removeAccount(KNNntpAccount *a)
   if(!a) a=c_urrentAccount;
   if(!a) return false;
 
-  QValueList<KNGroup*> lst;
-  if(knGlobals.folderManager()->unsentForAccount(a->id()) > 0) {
-    KMessageBox::sorry(knGlobals.topWidget, i18n("This account cannot be deleted since there are some unsent messages for it."));
+  KNGroup::List lst;
+  if ( knGlobals.folderManager()->unsentForAccount( a->id() ) > 0 ) {
+    KMessageBox::sorry( knGlobals.topWidget,
+      i18n("This account cannot be deleted since there are some unsent messages for it.") );
   }
-  else if(KMessageBox::warningContinueCancel(knGlobals.topWidget, i18n("Do you really want to delete this account?"),"",KGuiItem(i18n("&Delete"),"editdelete"))==KMessageBox::Continue) {
+  else if ( KMessageBox::warningContinueCancel ( knGlobals.topWidget,
+            i18n("Do you really want to delete this account?"), "", KGuiItem( i18n("&Delete"), "edit-delete") )
+            ==KMessageBox::Continue ) {
     lst = gManager->groupsOfAccount( a );
-    for ( QValueList<KNGroup*>::Iterator it = lst.begin(); it != lst.end(); ++it ) {
+    for ( KNGroup::List::Iterator it = lst.begin(); it != lst.end(); ++it ) {
       if ( (*it)->isLocked() ) {
-        KMessageBox::sorry(knGlobals.topWidget, i18n("At least one group of this account is currently in use.\nThe account cannot be deleted at the moment."));
+        KMessageBox::sorry( knGlobals.topWidget, i18n("At least one group of this account is currently in use.\n"
+            "The account cannot be deleted at the moment.") );
         return false;
       }
     }
-    for ( QValueList<KNGroup*>::Iterator it = lst.begin(); it != lst.end(); ++it )
+    for ( KNGroup::List::Iterator it = lst.begin(); it != lst.end(); ++it )
       gManager->unsubscribeGroup( (*it) );
 
     QDir dir(a->path());
     if (dir.exists()) {
-      const QFileInfoList *list = dir.entryInfoList();  // get list of matching files and delete all
-      if (list) {
-        QFileInfoListIterator it( *list );
-        while (it.current()) {
-          dir.remove(it.current()->fileName());
-          ++it;
-        }
+      QFileInfoList list = dir.entryInfoList();  // get list of matching files and delete all
+      QFileInfo it;
+      Q_FOREACH( it, list ) {
+        dir.remove(it.fileName());
       }
       dir.cdUp();                                       // directory should now be empty, deleting it
       dir.rmdir(QString("nntp.%1/").arg(a->id()));
@@ -183,7 +170,7 @@ bool KNAccountManager::removeAccount(KNNntpAccount *a)
     if(c_urrentAccount==a) setCurrentAccount(0);
 
     emit(accountRemoved(a));
-    mAccounts.remove( a );  // finally delete a
+    mAccounts.removeAll( a );  // finally delete a
     return true;
   }
 
@@ -244,9 +231,7 @@ void KNAccountManager::loadPasswordsAsync()
 
 void KNAccountManager::loadPasswords()
 {
-  s_mtp->readPassword();
-  QValueList<KNNntpAccount*>::Iterator it;
-  for ( it = mAccounts.begin(); it != mAccounts.end(); ++it )
+  for ( List::Iterator it = mAccounts.begin(); it != mAccounts.end(); ++it )
     (*it)->readPassword();
   emit passwordsChanged();
 }
@@ -265,7 +250,7 @@ KWallet::Wallet* KNAccountManager::wallet()
     mWallet = Wallet::openWallet( Wallet::NetworkWallet(),
                                   knGlobals.topWidget->topLevelWidget()->winId() );
   else
-    mWallet = Wallet::openWallet( Wallet::NetworkWallet() );
+    mWallet = Wallet::openWallet( Wallet::NetworkWallet(), 0 );
 
   if ( !mWallet ) {
     mWalletOpenFailed = true;
@@ -279,7 +264,9 @@ KWallet::Wallet* KNAccountManager::wallet()
 
 void KNAccountManager::prepareWallet()
 {
-  if ( mWallet && !mWallet->hasFolder("knode") )
+  if ( !mWallet )
+    return;
+  if ( !mWallet->hasFolder("knode") )
     mWallet->createFolder( "knode" );
   mWallet->setFolder( "knode" );
 }
