@@ -33,28 +33,37 @@
 #ifndef KNOTEHOSTDLG_H
 #define KNOTEHOSTDLG_H
 
-#include <kdialogbase.h>
+#include <kdialog.h>
 
-class KHistoryCombo;
+class KHistoryComboBox;
+class QTableView;
+class QModelIndex;
 
 
 /**
- * A dialog to request a hostname or IP address.
+ * A dialog that allows to select network service or request a hostname or IP address.
  */
-class KNoteHostDlg : public KDialogBase
+class KNoteHostDlg
+  : public KDialog
 {
-    Q_OBJECT
-public:
-    KNoteHostDlg( const QString& caption, QWidget *parent=0, const char *name=0 );
+  Q_OBJECT
+  public:
+    KNoteHostDlg( const QString &caption, QWidget *parent = 0 );
     ~KNoteHostDlg();
-
+    
     QString host() const;
-
-private slots:
+    /**
+     * Returns 0 if port was not specified
+     */
+    quint16 port() const;
+    
+  private slots:
     void slotTextChanged( const QString & );
-
-private:
-    KHistoryCombo *m_hostCombo;
+    void serviceSelected( const QModelIndex& );
+    
+  private:
+    KHistoryComboBox *m_hostCombo;
+    QTableView *m_servicesView;
 };
 
 #endif
