@@ -24,37 +24,35 @@
 #define KONTACT_MAINWINDOW_H
 
 #include "kontact_export.h"
-#include <kontactinterfaces/core.h>
 
-#include <kparts/part.h>
-#include <kparts/partmanager.h>
-
-#include <QPointer>
-
-class QFrame;
-class QSplitter;
-class QStackedWidget;
-
-class KConfig;
-class KPluginInfo;
-class KSqueezedTextLabel;
-class KHTMLPart;
-class KToolBarPopupAction;
+#include <KontactInterface/Core>
 
 namespace KPIM {
   class StatusbarProgressWidget;
 }
 
-namespace Kontact {
+namespace KontactInterface {
+  class Plugin;
+}
 
-class Plugin;
-class SidePaneBase;
-class AboutDialog;
+class KActionMenu;
+class KHTMLPart;
+class KPluginInfo;
+class KSqueezedTextLabel;
 
-typedef QList<Kontact::Plugin*> PluginList;
+class QFrame;
+class QSplitter;
+class QStackedWidget;
+
+typedef QList<KontactInterface::Plugin *> PluginList;
 typedef QList<QAction*> ActionPluginList;
 
-class KONTACT_EXPORT MainWindow : public Kontact::Core
+namespace Kontact {
+
+class AboutDialog;
+class SidePaneBase;
+
+class KONTACT_EXPORT MainWindow : public KontactInterface::Core
 {
   Q_OBJECT
   Q_CLASSINFO( "D-Bus Interface", "org.kde.kontact.KontactInterface" )
@@ -64,13 +62,14 @@ class KONTACT_EXPORT MainWindow : public Kontact::Core
     ~MainWindow();
 
     virtual PluginList pluginList() const { return mPlugins; }
-    void setActivePluginModule( const QString & );
+    void setInitialActivePluginModule( const QString & );
 
     static bool pluginActionWeightLessThan( const QAction *left, const QAction *right );
-    static bool pluginWeightLessThan( const Kontact::Plugin *left, const Kontact::Plugin *right );
+    static bool pluginWeightLessThan( const KontactInterface::Plugin *left,
+                                      const KontactInterface::Plugin *right );
 
   public slots:
-    virtual void selectPlugin( Kontact::Plugin *plugin );
+    virtual void selectPlugin( KontactInterface::Plugin *plugin );
     Q_SCRIPTABLE virtual void selectPlugin( const QString &pluginName );
     void slotActionTriggered();
 
@@ -90,7 +89,7 @@ class KONTACT_EXPORT MainWindow : public Kontact::Core
     void slotShowIntroduction();
     void showAboutDialog();
     void slotShowStatusMsg( const QString & );
-    void activatePluginModule();
+    void activateInitialPluginModule();
     void slotOpenUrl( const KUrl &url );
 
   private:
@@ -100,13 +99,13 @@ class KONTACT_EXPORT MainWindow : public Kontact::Core
     void saveSettings();
 
     bool isPluginLoaded( const KPluginInfo & );
-    Kontact::Plugin *pluginFromInfo( const KPluginInfo & );
+    KontactInterface::Plugin *pluginFromInfo( const KPluginInfo & );
     void loadPlugins();
     void unloadPlugins();
     void updateShortcuts();
     bool removePlugin( const KPluginInfo & );
-    void addPlugin( Kontact::Plugin *plugin );
-    void partLoaded( Kontact::Plugin *plugin, KParts::ReadOnlyPart *part );
+    void addPlugin( KontactInterface::Plugin *plugin );
+    void partLoaded( KontactInterface::Plugin *plugin, KParts::ReadOnlyPart *part );
     void setupActions();
     void showTip( bool );
     virtual bool queryClose();
@@ -127,11 +126,11 @@ class KONTACT_EXPORT MainWindow : public Kontact::Core
 
     QSplitter *mSplitter;
 
-    KToolBarPopupAction *mNewActions;
-    KToolBarPopupAction *mSyncActions;
+    KActionMenu *mNewActions;
+    KActionMenu *mSyncActions;
     SidePaneBase *mSidePane;
     QStackedWidget *mPartsStack;
-    Plugin *mCurrentPlugin;
+    KontactInterface::Plugin *mCurrentPlugin;
     KParts::PartManager *mPartManager;
     PluginList mPlugins;
     PluginList mDelayedPreload;
@@ -142,9 +141,10 @@ class KONTACT_EXPORT MainWindow : public Kontact::Core
     KSqueezedTextLabel *mStatusMsgLabel;
     KPIM::StatusbarProgressWidget *mLittleProgress;
 
-    QString mActiveModule;
+    QString mInitialActiveModule;
 
     QMap<QString, QPointer<QWidget> > mFocusWidgets;
+    QMap<KontactInterface::Plugin *, KAction *> mPluginAction;
 
     AboutDialog *mAboutDialog;
     bool mReallyClose;
@@ -153,7 +153,7 @@ class KONTACT_EXPORT MainWindow : public Kontact::Core
 
 }
 
-Q_DECLARE_METATYPE( Kontact::Plugin * )
+Q_DECLARE_METATYPE( KontactInterface::Plugin * )
 
 #endif
 // vim: sw=2 sts=2 et
