@@ -23,43 +23,33 @@
 */
 
 #include "koeditorfreebusy.h"
-#include "koprefs.h"
-#include "koglobals.h"
-#include "kogroupware.h"
 #include "freebusymanager.h"
 #include "freebusyurldialog.h"
-
-#ifndef KORG_NOKABC
-#include <kabc/addresseedialog.h>
-#include <kabc/vcardconverter.h>
-#include <libkdepim/addressesdialog.h>
-#include <libkdepim/addresseelineedit.h>
-#include <libkdepim/distributionlist.h>
-#include <kabc/stdaddressbook.h>
-#endif
-
-#include <kcal/event.h>
-#include <kcal/freebusy.h>
+#include "koglobals.h"
+#include "kogroupware.h"
+#include "koprefs.h"
 
 #include <kdgantt1/KDGanttView.h>
-#include <kdgantt1/KDGanttViewTaskItem.h>
 #include <kdgantt1/KDGanttViewSubwidgets.h>
+#include <kdgantt1/KDGanttViewTaskItem.h>
 
-#include <kpimutils/email.h>
+#include <akonadi/contact/contactgroupexpandjob.h>
+#include <akonadi/contact/contactgroupsearchjob.h>
+
+#include <KCal/FreeBusy>
+#include <KPIMUtils/Email>
 
-#include <klocale.h>
-#include <kiconloader.h>
-#include <kmessagebox.h>
 #include <KComboBox>
+#include <KLocale>
+#include <KMessageBox>
 
-#include <QToolTip>
-#include <QLayout>
-#include <QLabel>
-#include <QPushButton>
-#include <QFrame>
-#include <QHBoxLayout>
 #include <QBoxLayout>
-#include <QTimerEvent>
+#include <QDateTime>
+#include <QFrame>
+#include <QLabel>
+#include <KMenu>
+#include <QPushButton>
+#include <QToolTip>
 #include <QVBoxLayout>
 
 // The FreeBusyItem is the whole line for a given attendee.
@@ -243,61 +233,80 @@ KOEditorFreeBusy::KOEditorFreeBusy( int spacing, QWidget *parent )
   controlLayout->setSpacing( topLayout->spacing() );
   topLayout->addItem( controlLayout );
 
-  QString whatsThis = i18nc( "@info:whatsthis",
-                             "Sets the zoom level on the Gantt chart. "
-                             "'Hour' shows a range of several hours, "
-                             "'Day' shows a range of a few days, "
-                             "'Week' shows a range of a few months, "
-                             "and 'Month' shows a range of a few years, "
-                             "while 'Automatic' selects the range most "
-                             "appropriate for the current event or to-do." );
   QLabel *label = new QLabel( i18nc( "@label", "Scale: " ), this );
-  label->setWhatsThis( whatsThis );
   controlLayout->addWidget( label );
 
   scaleCombo = new KComboBox( this );
-  scaleCombo->setWhatsThis( whatsThis );
+  scaleCombo->setToolTip(
+    i18nc( "@info:tooltip", "Set the Gantt chart zoom level" ) );
+  scaleCombo->setWhatsThis(
+    i18nc( "@info:whatsthis",
+           "Select the Gantt chart zoom level from one of the following:<nl/>"
+           "'Hour' shows a range of several hours,<nl/>"
+           "'Day' shows a range of a few days,<nl/>"
+           "'Week' shows a range of a few months,<nl/>"
+           "and 'Month' shows a range of a few years,<nl/>"
+           "while 'Automatic' selects the range most "
+           "appropriate for the current event or to-do." ) );
   scaleCombo->addItem( i18nc( "@item:inlistbox range in hours", "Hour" ) );
   scaleCombo->addItem( i18nc( "@item:inlistbox range in days", "Day" ) );
   scaleCombo->addItem( i18nc( "@item:inlistbox range in weeks", "Week" ) );
   scaleCombo->addItem( i18nc( "@item:inlistbox range in months", "Month" ) );
   scaleCombo->addItem( i18nc( "@item:inlistbox range is computed automatically", "Automatic" ) );
   scaleCombo->setCurrentIndex( 0 ); // start with "hour"
-  connect( scaleCombo, SIGNAL( activated( int ) ),
-           SLOT( slotScaleChanged( int ) ) );
+  connect( scaleCombo, SIGNAL(activated(int)),
+           SLOT(slotScaleChanged(int)) );
   controlLayout->addWidget( scaleCombo );
 
   QPushButton *button = new QPushButton( i18nc( "@action:button", "Center on Start" ), this );
-  button->setWhatsThis( i18nc( "@info:whatsthis",
-                               "Centers the Gantt chart on the start time "
-                               "and day of this event." ) );
-  connect( button, SIGNAL( clicked() ), SLOT( slotCenterOnStart() ) );
+  button->setToolTip(
+    i18nc( "@info:tooltip",
+           "Center the Gantt chart on the event start date and time" ) );
+  button->setWhatsThis(
+    i18nc( "@info:whatsthis",
+           "Click this button to center the Gantt chart on the start "
+           "time and day of this event." ) );
+  connect( button, SIGNAL(clicked()), SLOT(slotCenterOnStart()) );
   controlLayout->addWidget( button );
 
   controlLayout->addStretch( 1 );
 
   button = new QPushButton( i18nc( "@action:button", "Pick Date" ), this );
-  button->setWhatsThis( i18nc( "@info:whatsthis",
-                               "Moves the event to a date and time when all "
-                               "the attendees are free." ) );
-  connect( button, SIGNAL( clicked() ), SLOT( slotPickDate() ) );
+  button->setToolTip(
+    i18nc( "@info:tooltip",
+           "Move the event to a date and time when all "
+           "attendees are available" ) );
+  button->setWhatsThis(
+    i18nc( "@info:whatsthis",
+           "Click this button to move the event to a date "
+           "and time when all the attendees have time "
+           "available in their Free/Busy lists." ) );
+  connect( button, SIGNAL(clicked()), SLOT(slotPickDate()) );
   controlLayout->addWidget( button );
 
   controlLayout->addStretch( 1 );
 
   button = new QPushButton( i18nc( "@action:button reload freebusy data", "Reload" ), this );
-  button->setWhatsThis( i18nc( "@info:whatsthis",
-                               "Reloads Free/Busy data for all attendees from "
-                               "the corresponding servers." ) );
+  button->setToolTip(
+    i18nc( "@info:tooltip",
+           "Reload Free/Busy data for all attendees" ) );
+  button->setWhatsThis(
+    i18nc( "@info:whatsthis",
+           "Pressing this button will cause the Free/Busy data for all "
+           "attendees to be reloaded from their corresponding servers." ) );
   controlLayout->addWidget( button );
-  connect( button, SIGNAL( clicked() ), SLOT( manualReload() ) );
+  connect( button, SIGNAL(clicked()), SLOT(manualReload()) );
 
   mGanttView = new KDGanttView( this, "mGanttView" );
-  mGanttView->setWhatsThis( i18nc( "@info:whatsthis",
-                                   "Shows the free/busy status of all attendees. "
-                                   "Double-clicking on an attendees entry in the "
-                                   "list will allow you to enter the location of "
-                                   "their Free/Busy Information." ) );
+  mGanttView->setToolTip(
+    i18nc( "@info:tooltip",
+           "Shows the Free/Busy status of all attendees" ) );
+  mGanttView->setWhatsThis(
+    i18nc( "@info:whatsthis",
+           "Shows the Free/Busy status of all attendees. "
+           "Double-clicking on an attendee's entry in the "
+           "list will allow you to enter the location of "
+           "their Free/Busy Information." ) );
   topLayout->addWidget( mGanttView );
   topLayout->setStretchFactor( mGanttView, 100 );
   // Remove the predefined "Task Name" column
@@ -331,29 +340,25 @@ KOEditorFreeBusy::KOEditorFreeBusy( int spacing, QWidget *parent )
   mEventRectangle->setColor( Qt::magenta );
   mGanttView->addIntervalBackgroundColor( mEventRectangle );
 
-  connect( mGanttView, SIGNAL ( timeIntervalSelected( const QDateTime &,
-                                                      const QDateTime & ) ),
-           mGanttView, SLOT( zoomToSelection( const QDateTime &,
-                                              const  QDateTime & ) ) );
-  connect( mGanttView, SIGNAL( lvItemDoubleClicked( KDGanttViewItem * ) ),
-           SLOT( editFreeBusyUrl( KDGanttViewItem * ) ) );
-  connect( mGanttView, SIGNAL( intervalColorRectangleMoved( const QDateTime&, const QDateTime& ) ),
-           this, SLOT( slotIntervalColorRectangleMoved( const QDateTime&, const QDateTime& ) ) );
+  connect( mGanttView, SIGNAL(timeIntervalSelected(const QDateTime &,const QDateTime &)),
+           mGanttView, SLOT(zoomToSelection(const QDateTime &,const  QDateTime &)) );
+  connect( mGanttView, SIGNAL(lvItemDoubleClicked(KDGanttViewItem *)),
+           SLOT(editFreeBusyUrl(KDGanttViewItem *)) );
+  connect( mGanttView, SIGNAL(intervalColorRectangleMoved(const QDateTime &,const QDateTime &)),
+           this, SLOT(slotIntervalColorRectangleMoved(const QDateTime &,const QDateTime &)) );
 
   connect( mGanttView, SIGNAL(lvSelectionChanged(KDGanttViewItem*)),
           this, SLOT(updateAttendeeInput()) );
-  connect( mGanttView, SIGNAL(lvItemLeftClicked(KDGanttViewItem*)),
-           this, SLOT(showAttendeeStatusMenu()) );
   connect( mGanttView, SIGNAL(lvItemRightClicked(KDGanttViewItem*)),
            this, SLOT(showAttendeeStatusMenu()) );
   connect( mGanttView, SIGNAL(lvMouseButtonClicked(int, KDGanttViewItem*, const QPoint&, int)),
            this, SLOT(listViewClicked(int, KDGanttViewItem*)) );
 
   FreeBusyManager *m = KOGroupware::instance()->freeBusyManager();
-  connect( m, SIGNAL( freeBusyRetrieved( KCal::FreeBusy *, const QString & ) ),
-           SLOT( slotInsertFreeBusy( KCal::FreeBusy *, const QString & ) ) );
+  connect( m, SIGNAL(freeBusyRetrieved(KCal::FreeBusy *,const QString &)),
+           SLOT(slotInsertFreeBusy(KCal::FreeBusy *,const QString &)) );
 
-  connect( &mReloadTimer, SIGNAL( timeout() ), SLOT( autoReload() ) );
+  connect( &mReloadTimer, SIGNAL(timeout()), SLOT(autoReload()) );
   mReloadTimer.setSingleShot( true );
 
   initEditWidgets( this, topLayout );
@@ -361,8 +366,8 @@ KOEditorFreeBusy::KOEditorFreeBusy( int spacing, QWidget *parent )
            SLOT(removeAttendee()) );
 
   slotOrganizerChanged( mOrganizerCombo->currentText() );
-  connect( mOrganizerCombo, SIGNAL( activated(const QString&) ),
-           this, SLOT( slotOrganizerChanged(const QString&) ) );
+  connect( mOrganizerCombo, SIGNAL(activated(const QString &)),
+           this, SLOT(slotOrganizerChanged(const QString &)) );
 
   //suppress the buggy consequences of clicks on the time header widget
   mGanttView->timeHeaderWidget()->installEventFilter( this );
@@ -700,6 +705,7 @@ void KOEditorFreeBusy::updateStatusSummary()
     case Attendee::Delegated:
     case Attendee::Completed:
     case Attendee::InProcess:
+    case Attendee::None:
       /* just to shut up the compiler */
       break;
     }
@@ -763,8 +769,9 @@ void KOEditorFreeBusy::editFreeBusyUrl( KDGanttViewItem *i )
 
   Attendee *attendee = item->attendee();
 
-  FreeBusyUrlDialog dialog( attendee, this );
-  dialog.exec();
+  QPointer<FreeBusyUrlDialog> dialog = new FreeBusyUrlDialog( attendee, this );
+  dialog->exec();
+  delete dialog;
 }
 
 void KOEditorFreeBusy::fillIncidence( Incidence *incidence )
@@ -778,23 +785,27 @@ void KOEditorFreeBusy::fillIncidence( Incidence *incidence )
     Q_ASSERT( attendee );
     /* Check if the attendee is a distribution list and expand it */
     if ( attendee->email().isEmpty() ) {
-      KPIM::DistributionList list =
-        KPIM::DistributionList::findByName( KABC::StdAddressBook::self(), attendee->name() );
-      if ( !list.isEmpty() ) {
+      Akonadi::ContactGroupSearchJob *job = new Akonadi::ContactGroupSearchJob();
+      job->setQuery( Akonadi::ContactGroupSearchJob::Name, attendee->name() );
+      job->exec();
+
+      const KABC::ContactGroup::List groups = job->contactGroups();
+      if ( !groups.isEmpty() ) {
         toBeDeleted.push_back( item ); // remove it once we are done expanding
-        KPIM::DistributionList::Entry::List entries = list.entries( KABC::StdAddressBook::self() );
-        KPIM::DistributionList::Entry::List::Iterator it( entries.begin() );
-        while ( it != entries.end() ) {
-          KPIM::DistributionList::Entry &e = ( *it );
-          ++it;
+        Akonadi::ContactGroupExpandJob *expandJob =
+          new Akonadi::ContactGroupExpandJob( groups.first() );
+        expandJob->exec();
+
+        const KABC::Addressee::List contacts = expandJob->contacts();
+        foreach ( const KABC::Addressee &contact, contacts ) {
           // this calls insertAttendee, which appends
-          insertAttendeeFromAddressee( e.addressee, attendee );
+          insertAttendeeFromAddressee( contact, attendee );
           // TODO: duplicate check, in case it was already added manually
         }
       }
     } else {
       bool skip = false;
-      if ( attendee->email().endsWith( "example.net" ) ) {
+      if ( attendee->email().endsWith( QLatin1String( "example.net" ) ) ) {
         if ( KMessageBox::warningYesNo(
                this,
                i18nc( "@info",
@@ -846,20 +857,23 @@ void KOEditorFreeBusy::removeAttendee()
   }
 
   FreeBusyItem *nextSelectedItem = static_cast<FreeBusyItem*>( item->nextSibling() );
-  if( mGanttView->childCount() == 1 )
-      nextSelectedItem = 0;
-  if( mGanttView->childCount() > 1 && item == mGanttView->lastItem() )
-      nextSelectedItem = static_cast<FreeBusyItem*>(  mGanttView->firstChild() );
+  if( mGanttView->childCount() == 1 ) {
+    nextSelectedItem = 0;
+  }
+  if( mGanttView->childCount() > 1 && item == mGanttView->lastItem() ) {
+    nextSelectedItem = static_cast<FreeBusyItem*>( mGanttView->firstChild() );
+  }
 
   Attendee *delA = new Attendee( item->attendee()->name(), item->attendee()->email(),
                                  item->attendee()->RSVP(), item->attendee()->status(),
                                  item->attendee()->role(), item->attendee()->uid() );
-  mdelAttendees.append( delA );
+  mDelAttendees.append( delA );
   delete item;
 
   updateStatusSummary();
-  if( nextSelectedItem )
-      mGanttView->setSelected( nextSelectedItem, true );
+  if( nextSelectedItem ) {
+    mGanttView->setSelected( nextSelectedItem, true );
+  }
   updateAttendeeInput();
   emit updateAttendeeSummary( mGanttView->childCount() );
 }
@@ -891,33 +905,53 @@ void KOEditorFreeBusy::changeStatusForMe( KCal::Attendee::PartStat status )
 
 void KOEditorFreeBusy::showAttendeeStatusMenu()
 {
-  if ( mGanttView->mapFromGlobal( QCursor::pos() ).x() > 22 ) {
+  if( !currentAttendee() || mGanttView->selectedItem() == hasExampleAttendee() ) {
+     return;
+  }
+
+  KMenu *menu = new KMenu( 0 );
+  QAction *needsaction =
+    menu->addAction( KOGlobals::self()->smallIcon( "help-about" ),
+                     Attendee::statusName( Attendee::NeedsAction ) );
+  QAction *accepted =
+    menu->addAction( KOGlobals::self()->smallIcon( "dialog-ok-apply" ),
+                     Attendee::statusName( Attendee::Accepted ) );
+  QAction *declined =
+    menu->addAction( KOGlobals::self()->smallIcon( "dialog-cancel" ),
+                     Attendee::statusName( Attendee::Declined ) );
+  QAction *tentative =
+    menu->addAction( KOGlobals::self()->smallIcon( "dialog-ok" ),
+                     Attendee::statusName( Attendee::Tentative ) );
+  QAction *delegated =
+    menu->addAction( KOGlobals::self()->smallIcon( "mail-forward" ),
+                     Attendee::statusName( Attendee::Delegated ) );
+  QAction *completed =
+    menu->addAction( KOGlobals::self()->smallIcon( "mail-mark-read" ),
+                     Attendee::statusName( Attendee::Completed ) );
+  QAction *inprocess =
+    menu->addAction( KOGlobals::self()->smallIcon( "help-about" ),
+                     Attendee::statusName( Attendee::InProcess ) );
+  QAction *ret = menu->exec( QCursor::pos() );
+  delete menu;
+  if ( ret == needsaction ) {
+    currentAttendee()->setStatus( Attendee::NeedsAction );
+  } else if ( ret == accepted ) {
+    currentAttendee()->setStatus( Attendee::Accepted );
+  } else if ( ret == declined ) {
+    currentAttendee()->setStatus( Attendee::Declined );
+  } else if ( ret == tentative ) {
+    currentAttendee()->setStatus( Attendee::Tentative );
+  } else if ( ret == delegated ) {
+    currentAttendee()->setStatus( Attendee::Delegated );
+  } else if ( ret == completed ) {
+    currentAttendee()->setStatus( Attendee::Completed );
+  } else if ( ret == inprocess ) {
+    currentAttendee()->setStatus( Attendee::InProcess );
+  } else {
     return;
   }
-  if( !currentAttendee() )
-      return;
-  QPopupMenu popup;
-  popup.insertItem( KOGlobals::self()->smallIcon( "help-about" ),
-                    Attendee::statusName( Attendee::NeedsAction ), Attendee::NeedsAction );
-  popup.insertItem( KOGlobals::self()->smallIcon( "dialog-ok-apply" ),
-                    Attendee::statusName( Attendee::Accepted ), Attendee::Accepted );
-  popup.insertItem( KOGlobals::self()->smallIcon( "dialog-cancel" ),
-                    Attendee::statusName( Attendee::Declined ), Attendee::Declined );
-  popup.insertItem( KOGlobals::self()->smallIcon( "dialog-ok" ),
-                    Attendee::statusName( Attendee::Tentative ), Attendee::Tentative );
-  popup.insertItem( KOGlobals::self()->smallIcon( "mail-forward" ),
-                    Attendee::statusName( Attendee::Delegated ), Attendee::Delegated );
-  popup.insertItem( KOGlobals::self()->smallIcon( "mail-mark-read" ),
-                    Attendee::statusName( Attendee::Completed ), Attendee::Completed );
-  popup.insertItem( KOGlobals::self()->smallIcon( "help-about" ),
-                    Attendee::statusName( Attendee::InProcess ), Attendee::InProcess );
-  popup.setItemChecked( currentAttendee()->status(), true );
-  int status = popup.exec( QCursor::pos() );
-  if ( status >= 0 ) {
-    currentAttendee()->setStatus( static_cast<Attendee::PartStat>( status ) );
-    updateCurrentItem();
-    updateAttendeeInput();
-  }
+  updateCurrentItem();
+  updateAttendeeInput();
 }
 
 void KOEditorFreeBusy::listViewClicked( int button, KDGanttViewItem *item )
@@ -962,8 +996,10 @@ void KOEditorFreeBusy::slotOrganizerChanged( const QString &newOrganizer )
   if ( currentOrganizerAttendee ) {
     answer = KMessageBox::questionYesNo(
       this,
-      i18n( "You are changing the organiser of this event, who is also "
-            "attending, do you want to change that attendee as well?" ) );
+      i18nc( "@option",
+             "You are changing the organizer of this event. "
+             "Since the organizer is also attending this event, would you "
+             "like to change the corresponding attendee as well?" ) );
   } else {
     answer = KMessageBox::Yes;
   }
