@@ -214,51 +214,6 @@ class KOPrefsDialogTime : public KPrefsModule
 
       QGridLayout *datetimeLayout = new QGridLayout( datetimeGroupBox );
 
-      KHBox *timeZoneBox = new KHBox( regionalPage );
-      datetimeLayout->addWidget( timeZoneBox, 0, 0, 1, 2 );
-
-      QLabel *timeZoneLabel = new QLabel( i18nc( "@label", "Time zone:" ), timeZoneBox );
-      QString whatsThis = i18nc( "@info:whatsthis",
-                                 "Select your time zone from the list of "
-                                 "locations on this drop down box. If your "
-                                 "city is not listed, select one which shares "
-                                 "the same time zone. KOrganizer will "
-                                 "automatically adjust for daylight savings." );
-      timeZoneLabel->setWhatsThis( whatsThis );
-      mTimeZoneCombo = new KComboBox( timeZoneBox );
-
-      connect( mTimeZoneCombo, SIGNAL(activated(int)), SLOT(slotWidChanged()) );
-
-      QString sCurrentlySet(
-        i18nc( "@item:inlistbox unknown timezone", "Unknown" ) );
-      KTimeZone zone = KSystemTimeZones::local();
-      if ( zone.isValid() ) {
-        sCurrentlySet = zone.name();
-      }
-      // Read all system time zones
-      QStringList list;
-      const KTimeZones::ZoneMap timezones = KSystemTimeZones::zones();
-      for ( KTimeZones::ZoneMap::ConstIterator it = timezones.constBegin();
-            it != timezones.constEnd();  ++it ) {
-        list.append( i18n( it.key().toUtf8() ) );
-        tzonenames << it.key();
-      }
-      list.sort();
-      mTimeZoneCombo->addItem(
-        i18nc( "@item:inlistbox no timezone selected", "[No selection]" ) );
-      mTimeZoneCombo->addItems( list );
-
-      // find the currently set time zone and select it
-      int nCurrentlySet = 0;
-      for ( int i = 0; i < mTimeZoneCombo->count(); ++i ) {
-        if ( mTimeZoneCombo->itemText(i) == sCurrentlySet ) {
-          nCurrentlySet = i;
-          break;
-        }
-      }
-      mTimeZoneCombo->setCurrentIndex( nCurrentlySet );
-      mTimeZoneCombo->setWhatsThis( whatsThis );
-
       KPrefsWidTime *dayBegins =
         addWidTime( KOPrefs::instance()->dayBeginsItem(), regionalPage );
       datetimeLayout->addWidget( dayBegins->label(), 1, 0 );
@@ -322,7 +277,7 @@ class KOPrefsDialogTime : public KPrefsModule
       }
 
       QGroupBox *workingHoursGroupBox =
-        new QGroupBox( i18nc( "@title:group", "Working Hours" ), regionalPage );
+        new QGroupBox( i18nc( "@title:group", "Working Period" ), regionalPage );
       regionalLayout->addWidget( workingHoursGroupBox, 2, 0 );
 
       QBoxLayout *workingHoursLayout = new QVBoxLayout( workingHoursGroupBox );
@@ -432,6 +387,11 @@ class KOPrefsDialogTime : public KPrefsModule
         i18nc( "@item:inlistbox reminder time units in days", "day(s)" ) );
       remindersLayout->addWidget( mReminderUnitsCombo, 0, 2 );
 
+      remindersLayout->addWidget(
+        addWidBool( KOPrefs::instance()->defaultEventRemindersItem() )->checkBox(), 1, 0 );
+      remindersLayout->addWidget(
+        addWidBool( KOPrefs::instance()->defaultTodoRemindersItem() )->checkBox(), 2, 0 );
+
       defaultLayout->setRowStretch( 3, 1 );
       load();
     }
@@ -439,11 +399,6 @@ class KOPrefsDialogTime : public KPrefsModule
   protected:
     void usrReadConfig()
     {
-      KTimeZone tz = KOPrefs::instance()->timeSpec().timeZone();
-      if ( tz.isValid() ) {
-        setCombo( mTimeZoneCombo, i18n( tz.name().toUtf8() ) );
-      }
-
       mReminderTimeSpin->setValue( KOPrefs::instance()->mReminderTime );
       mReminderUnitsCombo->setCurrentIndex( KOPrefs::instance()->mReminderTimeUnits );
       for ( int i = 0; i < 7; ++i ) {
@@ -453,20 +408,6 @@ class KOPrefsDialogTime : public KPrefsModule
 
     void usrWriteConfig()
     {
-      // Find untranslated selected zone
-      QString selectedZone = mTimeZoneCombo->currentText();
-      QStringList::Iterator tz;
-      for ( tz = tzonenames.begin(); tz != tzonenames.end(); ++tz ) {
-        if ( selectedZone == i18n( (*tz).toUtf8() ) ) {
-          selectedZone = *tz;
-          break;
-        }
-      }
-      KTimeZone zone = KSystemTimeZones::zone( selectedZone );
-      if ( zone.isValid() ) {
-        KOPrefs::instance()->setTimeSpec(zone);
-      }
-
       KOPrefs::instance()->mHolidays = ( mHolidayCombo->currentIndex() == 0 ) ?  // (None)
                                        QString() :
                                        mRegionMap[mHolidayCombo->currentText()];
@@ -501,7 +442,6 @@ class KOPrefsDialogTime : public KPrefsModule
     }
 
   private:
-    KComboBox    *mTimeZoneCombo;
     QStringList   tzonenames;
     KComboBox    *mHolidayCombo;
     QMap<QString,QString> mRegionMap;
@@ -639,8 +579,8 @@ class KOPrefsDialogViews : public KPrefsModule
       // GroupBox: Views->Month View->Display Options
       QVBoxLayout *mdisplayLayout = new QVBoxLayout;
       QGroupBox *mdisplayBox = new QGroupBox( i18nc( "@title:group", "Display Options" ) );
-      mdisplayLayout->addWidget(
-        addWidBool( KOPrefs::instance()->enableMonthScrollItem() )->checkBox() );
+      /*mdisplayLayout->addWidget(
+        addWidBool( KOPrefs::instance()->enableMonthScrollItem() )->checkBox() );*/
       mdisplayLayout->addWidget(
         addWidBool( KOPrefs::instance()->enableMonthItemIconsItem() )->checkBox() );
       mdisplayLayout->addWidget(
@@ -795,8 +735,8 @@ KOPrefsDialogColorsAndFonts::KOPrefsDialogColorsAndFonts( const KComponentData &
   mResourceCombo = new KComboBox( resourceGroup );
   mResourceCombo->setWhatsThis(
     i18nc( "@info:whatsthis",
-           "Select here resource you want to modify. "
-           "You can change the selected resource color using "
+           "Select the calendar you want to modify. "
+           "You can change the selected calendar color using "
            "the button below." ) );
   connect( mResourceCombo, SIGNAL(activated(int)), SLOT(updateResourceColor()) );
   resourceLayout->addWidget( mResourceCombo );
@@ -804,7 +744,7 @@ KOPrefsDialogColorsAndFonts::KOPrefsDialogColorsAndFonts( const KComponentData &
   mResourceButton = new KColorButton( resourceGroup );
   mResourceButton->setWhatsThis(
     i18nc( "@info:whatsthis",
-           "Choose here the color of the resource selected "
+           "Choose here the color of the calendar selected "
            "using the combo box above." ) );
   connect( mResourceButton, SIGNAL(changed(const QColor &)), SLOT(setResourceColor()) );
   resourceLayout->addWidget( mResourceButton );
@@ -931,20 +871,25 @@ void KOPrefsDialogColorsAndFonts::updateResources()
 
 void KOPrefsDialogColorsAndFonts::setResourceColor()
 {
-  mResourceDict.insert( mResourceIdentifier[mResourceCombo->currentIndex()],
-                        mResourceButton->color() );
+  if ( mResourceCombo->currentIndex() > 0 ) {
+    mResourceDict.insert( mResourceIdentifier[mResourceCombo->currentIndex()],
+                          mResourceButton->color() );
+  }
   slotWidChanged();
 }
 
 void KOPrefsDialogColorsAndFonts::updateResourceColor()
 {
-  QString res = mResourceIdentifier[mResourceCombo->currentIndex()];
-  QColor color = mCategoryDict.value( res );
-  if ( !color.isValid() )  {
-    color = KOPrefs::instance()->resourceColor( res );
-  }
-  if ( color.isValid() ) {
-    mResourceButton->setColor( color );
+  if ( mResourceCombo->currentIndex() > 0 ) {
+    QString res = mResourceIdentifier[mResourceCombo->currentIndex()];
+
+    QColor color = mCategoryDict.value( res );
+    if ( !color.isValid() )  {
+      color = KOPrefs::instance()->resourceColor( res );
+    }
+    if ( color.isValid() ) {
+      mResourceButton->setColor( color );
+    }
   }
 }
 extern "C"
@@ -998,6 +943,7 @@ KOPrefsDialogGroupScheduling::KOPrefsDialogGroupScheduling( const KComponentData
   aMailsLabel->setWhatsThis( whatsThis );
   topLayout->addWidget( aMailsLabel, 3, 0, 1, 2 );
   mAMails = new Q3ListView( topFrame );
+  mAMails->setResizeMode(Q3ListView::LastColumn);
   mAMails->setWhatsThis( whatsThis );
 
   mAMails->addColumn( i18nc( "@title:column email addresses", "Email" ), 300 );
@@ -1044,8 +990,9 @@ KOPrefsDialogGroupScheduling::KOPrefsDialogGroupScheduling( const KComponentData
 void KOPrefsDialogGroupScheduling::usrReadConfig()
 {
   mAMails->clear();
-  for ( QStringList::Iterator it = KOPrefs::instance()->mAdditionalMails.begin();
-            it != KOPrefs::instance()->mAdditionalMails.end(); ++it ) {
+  QStringList::const_iterator begin( KOPrefs::instance()->mAdditionalMails.constBegin() );
+  QStringList::const_iterator end( KOPrefs::instance()->mAdditionalMails.constEnd() );
+  for ( QStringList::const_iterator it = begin; it != end; ++it ) {
     Q3ListViewItem *item = new Q3ListViewItem(mAMails);
     item->setText( 0, *it );
     mAMails->insertItem( item );
