@@ -66,6 +66,7 @@ KOViewManager::KOViewManager( CalendarView *mainView )
   mAgendaViewTabs = 0;
   mTimeSpentView = 0;
   mMonthView = 0;
+  mAgendaMode = AGENDA_NONE;
 }
 
 KOViewManager::~KOViewManager()
@@ -93,14 +94,34 @@ void KOViewManager::readSettings( KConfig *config )
     showJournalView();
   } else if ( view == QLatin1String( "Todo" ) ) {
     showTodoView();
-  } else if ( view == "Timeline" ) {
+  } else if ( view == QLatin1String( "Timeline" ) ) {
     showTimeLineView();
-  } else if ( view == "TimeSpent" ) {
+  } else if ( view == QLatin1String( "TimeSpent" ) ) {
     showTimeSpentView();
-  } else if ( view == "Month" ) {
+  } else if ( view == QLatin1String( "Month" ) ) {
     showMonthView();
   } else {
-    showAgendaView();
+    mAgendaMode = AgendaMode( generalConfig.readEntry( "Agenda Mode", int( AGENDA_OTHER ) ) );
+
+    switch ( mAgendaMode ) {
+      case AGENDA_WORK_WEEK:
+        showWorkWeekView();
+        break;
+      case AGENDA_WEEK:
+        showWeekView();
+        break;
+      case AGENDA_NEXTX:
+        showNextXView();
+        break;
+      case AGENDA_DAY:
+        showDayView();
+        break;
+      case AGENDA_NONE:
+        // Someone has been playing with the config file.
+      default:
+        mAgendaMode = AGENDA_OTHER;
+        showAgendaView();
+    } 
   }
 }
 
@@ -110,21 +131,22 @@ void KOViewManager::writeSettings( KConfig *config )
 
   QString view;
   if ( mCurrentView == mWhatsNextView ) {
-    view = "WhatsNext";
+    view = QLatin1String( "WhatsNext" );
   } else if ( mCurrentView == mListView ) {
-    view = "List";
+    view = QLatin1String( "List" );
   } else if ( mCurrentView == mJournalView ) {
-    view = "Journal";
+    view = QLatin1String( "Journal" );
   } else if ( mCurrentView == mTodoView ) {
-    view = "Todo";
+    view = QLatin1String( "Todo" );
   } else if ( mCurrentView == mTimelineView ) {
-    view = "Timeline";
+    view = QLatin1String( "Timeline" );
   } else if ( mCurrentView == mTimeSpentView ) {
-    view = "TimeSpent";
+    view = QLatin1String( "TimeSpent" );
   } else if ( mCurrentView == mMonthView ) {
-    view = "Month";
+    view = QLatin1String( "Month" );
   } else {
-    view = "Agenda";
+    view = QLatin1String( "Agenda" );
+    generalConfig.writeEntry( "Agenda Mode", int( mAgendaMode ) );
   }
 
   generalConfig.writeEntry( "Current View", view );
@@ -158,7 +180,7 @@ void KOViewManager::showView( KOrg::BaseView *view )
   }
 
   raiseCurrentView();
-  mMainView->processIncidenceSelection( 0 );
+  mMainView->processIncidenceSelection( 0, QDate() );
   mMainView->updateView();
   mMainView->adaptNavigationUnits();
 }
@@ -227,8 +249,8 @@ void KOViewManager::connectView( KOrg::BaseView *view )
   }
 
   // selecting an incidence
-  connect( view, SIGNAL(incidenceSelected(Incidence *)),
-           mMainView, SLOT(processMainViewSelection(Incidence *)) );
+  connect( view, SIGNAL(incidenceSelected(Incidence *,const QDate &)),
+           mMainView, SLOT(processMainViewSelection(Incidence *,const QDate &)) );
 
   // showing/editing/deleting an incidence. The calendar view takes care of the action.
   connect( view, SIGNAL(showIncidenceSignal(Incidence *)),
@@ -247,6 +269,10 @@ void KOViewManager::connectView( KOrg::BaseView *view )
            mMainView, SLOT(toggleAlarm(Incidence *)) );
   connect( view, SIGNAL(toggleTodoCompletedSignal(Incidence *)),
            mMainView, SLOT(toggleTodoCompleted(Incidence *)) );
+  connect( view, SIGNAL(copyIncidenceToResourceSignal(Incidence *,const QString &)),
+           mMainView, SLOT(copyIncidenceToResource(Incidence *,const QString &)) );
+  connect( view, SIGNAL(moveIncidenceToResourceSignal(Incidence *,const QString &)),
+           mMainView, SLOT(moveIncidenceToResource(Incidence *,const QString &)) );
   connect( view, SIGNAL(dissociateOccurrencesSignal(Incidence *,const QDate &)),
            mMainView, SLOT(dissociateOccurrences(Incidence *,const QDate &)) );
 
@@ -448,6 +474,7 @@ void KOViewManager::showAgendaView()
 
 void KOViewManager::showDayView()
 {
+  mAgendaMode = AGENDA_DAY;
   QDate date = mMainView->activeDate();
   showAgendaView();
   mMainView->dateNavigator()->selectDate( date );
@@ -456,16 +483,21 @@ void KOViewManager::showDayView()
 void KOViewManager::showWorkWeekView()
 {
   if ( KOGlobals::self()->getWorkWeekMask() != 0 ) {
+    mAgendaMode = AGENDA_WORK_WEEK;
     QDate date = mMainView->activeDate();
     showAgendaView();
     mMainView->dateNavigator()->selectWorkWeek( date );
   } else {
-    // TODO: tell the user he must configure work days
+    KMessageBox::sorry(
+      mMainView,
+      i18n( "Unable to display the work week view since there are no work days configured. "
+            "Please properly configure at least 1 work day in the Time and Date preferences." ) );
   }
 }
 
 void KOViewManager::showWeekView()
 {
+  mAgendaMode = AGENDA_WEEK;
   QDate date = mMainView->activeDate();
   showAgendaView();
   mMainView->dateNavigator()->selectWeek( date );
@@ -473,6 +505,7 @@ void KOViewManager::showWeekView()
 
 void KOViewManager::showNextXView()
 {
+  mAgendaMode = AGENDA_NEXTX;
   showAgendaView();
   mMainView->dateNavigator()->selectDates( QDate::currentDate(),
                                            KOPrefs::instance()->mNextXDays );
@@ -543,7 +576,7 @@ QDate KOViewManager::currentSelectionDate()
 {
   QDate qd;
   if ( mCurrentView ) {
-    DateList qvl = mCurrentView->selectedDates();
+    DateList qvl = mCurrentView->selectedIncidenceDates();
     if ( !qvl.isEmpty() ) {
       qd = qvl.first();
     }
@@ -583,3 +616,14 @@ void KOViewManager::setUpdateNeeded()
     mAgendaSideBySideView->setUpdateNeeded();
   }
 }
+
+void KOViewManager::updateMultiCalendarDisplay()
+{
+  if ( mCurrentView == mAgendaView            ||
+       mCurrentView == mAgendaSideBySideView  ||
+       ( mAgendaViewTabs && mCurrentView == mAgendaViewTabs->currentWidget() ) ) {
+    showAgendaView();
+  } else {
+    updateView();
+  }
+}
