@@ -15,24 +15,24 @@
 
     You should have received a copy of the GNU Library General Public License
     along with this library; see the file COPYING.LIB.  If not, write to
-    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-    Boston, MA 02111-1307, USA.
+    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+    Boston, MA 02110-1301, USA.
 */
 
-#ifndef __LIBKDEPIM_PLUGINLOADERBASE_H__
-#define __LIBKDEPIM_PLUGINLOADERBASE_H__
+#ifndef KDEPIM_PLUGINLOADERBASE_H
+#define KDEPIM_PLUGINLOADERBASE_H
 
-#include <qstring.h>
-#include <qmap.h>
+#include "kdepim_export.h"
 
-#include <kdepimmacros.h>
+#include <QString>
+#include <QMap>
+#include <KLibrary>
 
-class KLibrary;
 class QStringList;
 
 namespace KPIM {
 
-  class KDE_EXPORT PluginMetaData {
+  class KDEPIM_EXPORT PluginMetaData {
   public:
     PluginMetaData() {}
     PluginMetaData( const QString & lib, const QString & name,
@@ -42,10 +42,10 @@ namespace KPIM {
     QString library;
     QString nameLabel;
     QString descriptionLabel;
-    mutable bool loaded;
+    bool loaded;
   };
 
-  class KDE_EXPORT PluginLoaderBase {
+  class KDEPIM_EXPORT PluginLoaderBase {
   protected:
     PluginLoaderBase();
     virtual ~PluginLoaderBase();
@@ -64,18 +64,18 @@ namespace KPIM {
   protected:
     /** Rescans the plugin directory to find any newly installed
 	plugins. Extend this method in subclasses to add any
-	builtins. Subclasses must call this explicitely. It's not
+	builtins. Subclasses must call this explicitly. It's not
 	called for them in the constructor.
     **/
     void doScan( const char * path );
 
     /** Returns a pointer to symbol @p main_func in the library that
         implements the plugin of type @p type */
-    void * mainFunc( const QString & type, const char * main_func ) const;
+    KLibrary::void_function_ptr mainFunc( const QString & type, const char * main_func ) const;
 
   private:
     const KLibrary * openLibrary( const QString & libName ) const;
-    QMap< QString, PluginMetaData > mPluginMap;
+    mutable QMap< QString, PluginMetaData > mPluginMap;
 
     class Private;
     Private * d;
